@@ -1,7 +1,7 @@
 /* boot.c  -  Boot image composition */
 /*
 Copyright 1992-1997 Werner Almesberger.
-Copyright 1999-2002 John Coffman.
+Copyright 1999-2003 John Coffman.
 All rights reserved.
 
 Licensed under the terms contained in the file 'COPYING' in the 
@@ -21,9 +21,10 @@ source directory.
 #include <sys/stat.h>
 
 #include "config.h"
-#include "common.h"
 #include "lilo.h"
+#include "common.h"
 #include "geometry.h"
+#include "device.h"
 #include "cfg.h"
 #include "map.h"
 #include "partition.h"
@@ -199,6 +200,7 @@ void do_map_drive(void)
  
 char *boot_mbr(const char *boot, int table)
 {
+#if 0
     char *part, *npart, *endptr;
     int i, j, k;
     
@@ -236,6 +238,28 @@ char *boot_mbr(const char *boot, int table)
     else 
 	k = 0;
 
+#else
+    struct stat st;
+    dev_t dev;
+    DEVICE d;
+    int mask, k;
+    char *npart = NULL;
+    
+    k = 0;
+    if (stat(boot,&st) < 0) die("stat %s: %s",boot,strerror(errno));
+    dev = S_ISREG(st.st_mode) ? st.st_dev : st.st_rdev;
+    if ( (mask = has_partitions(dev)) ) {
+	k = dev & ~mask;
+	k = !table ? 1 : k>=1 && k<=4;
+	if (k) {
+	    dev_open(&d, dev&mask, -1);	/* bypass any open */
+	    npart = stralloc(d.name); 
+	    dev_close(&d);
+	}
+    }
+    
+
+#endif
     if (verbose>=3) {
         printf("Name: %s  yields MBR: %s  (with%s primary partition check)\n",
            boot, k ? npart : "(NULL)", table ? "" : "out");
@@ -333,6 +357,16 @@ void boot_other(char *loader,char *boot,char *part,IMAGE_DESCR *descr)
 	if (magic != BOOT_SIGNATURE)
 	    die("First sector of %s doesn't have a valid boot signature",boot);
     }
+    if (cfg_get_flag(cf_other,"master-boot")) {
+	curr_drv_map += 2;	/* add 2 spaces */
+	if (curr_drv_map >= DRVMAP_SIZE)
+	    cfg_error("Too many drive mappings (more than %d)",DRVMAP_SIZE);
+	if (verbose > 1)
+	    printf("  Swapping BIOS boot drive with 0/0x80, as needed\n");
+	for (i=curr_drv_map-1; i>1; i--) drv_map[i] = drv_map[i-2];
+	drv_map[1] = drv_map[0] = 0xFFFF;	/* reserve 2 slots */
+    }
+
     memset(buff,0,sizeof(buff));
 #ifndef LCF_BUILTIN
     if ((l_fd = open(loader,O_RDONLY)) < 0)
@@ -397,7 +431,13 @@ void boot_other(char *loader,char *boot,char *part,IMAGE_DESCR *descr)
     memcpy(pos,drv_map,sizeof(drv_map));
     memcpy(pos+sizeof(drv_map),prt_map,sizeof(prt_map)-2);
     map_add_zero();
+#if 0
     for (i = 0; i < SETUPSECS-1; i++) map_add_sector(&buff[i]);
+#else
+    size = (size + SECTOR_SIZE - 1) / SECTOR_SIZE;
+    for (i = 0; i < size; i++) map_add_sector(&buff[i]);
+    for (; i < SETUPSECS-1; i++) map_add_zero();
+#endif
     map_add(&geo,0,1);
     (void) map_end_section(&descr->start,SETUPSECS+SPECIAL_SECTORS);
 	/* size is known */
