@@ -6,8 +6,6 @@
 #ifndef BOOT_H
 #define BOOT_H
 
-#include "lilo.h"
-
 
 void boot_image(char *spec,IMAGE_DESCR *descr);
 
@@ -23,8 +21,10 @@ void boot_unstripped(char *boot,char *setup,char *kernel,IMAGE_DESCR *descr);
 /* Maps an unstripped kernel image as the boot image. The setup (without the
    header) is prepended. */
 
-char *boot_table(char *boot);
-/* derive default TABLE= from OTHER=, if possible */
+char *boot_mbr(const char *boot, int table);
+/* derive name of MBR from partition name; check for primary partition
+   if table==1
+ */
 
 void boot_other(char *loader,char *boot,char *part,IMAGE_DESCR *descr);
 
