@@ -1,7 +1,7 @@
 /* bsect.c  -  Boot sector handling */
 /*
 Copyright 1992-1998 Werner Almesberger.
-Copyright 1999-2002 John Coffman.
+Copyright 1999-2003 John Coffman.
 All rights reserved.
 
 Licensed under the terms contained in the file 'COPYING' in the 
@@ -22,9 +22,6 @@ source directory.
 #define	_I386_STATFS_H	/* two versions of statfs is not good ... */
 #endif
 
-#include <linux/fs.h>
-#include <linux/hdreg.h>
-#include <linux/fd.h>
 #include <string.h>
 #include <stdlib.h>
 #include <ctype.h>
@@ -32,9 +29,10 @@ source directory.
 #include <assert.h>
 
 #include "config.h"
-#include "common.h"
-#include "cfg.h"
 #include "lilo.h"
+#include "common.h"
+#include "raid.h"
+#include "cfg.h"
 #include "device.h"
 #include "geometry.h"
 #include "map.h"
@@ -43,6 +41,7 @@ source directory.
 #include "boot.h"
 #include "bsect.h"
 #include "bitmap.h"
+#include "probe.h"
 #include "loader.h"
 #include "edit.h"
 
@@ -57,6 +56,7 @@ source directory.
 int boot_dev_nr;
 
 static BOOT_SECTOR bsect,bsect_orig;
+static MENUTABLE menuparams;
 static DESCR_SECTORS descrs;
 static char secondary_map[SECTOR_SIZE];
 static unsigned char table[SECTOR_SIZE];	/* keytable & params */
@@ -70,6 +70,9 @@ static int fallbacks = 0;
 static unsigned short stage_flags;
 static int image_menu_space = MAX_IMAGES;
 static char *getval_user;
+static BOOT_PARAMS_2 param2;
+static off_t here2;		/* sector address of second stage loader */
+static int adapter = -1;	/* video adapter in use */
 
 typedef struct Pass {
     long crc[MAX_PW_CRC];
@@ -80,13 +83,12 @@ typedef struct Pass {
 
 static PASSWORD *pwsave = NULL;
 
-#ifdef LCF_BUILTIN
 BUILTIN_FILE *select_loader(void)
 {
     BUILTIN_FILE *loader = &Third;	/* MENU interface is the default */
     char *install = cfg_get_strg(cf_options,"install");
     char *bitmap  = cfg_get_strg(cf_options,"bitmap");
-    
+
     if (!install) {
 	if (bitmap) loader = &Bitmap;
     } 
@@ -94,9 +96,21 @@ BUILTIN_FILE *select_loader(void)
     else if (strstr(install,"menu")) loader = &Third;
     else if (bitmap) loader = &Bitmap;   /* menu interface (MDA, EGA, VGA) */
         
+    adapter = get_video();	/* from probe.c */
+    if (!nowarn) {
+	if (adapter <= VIDEO_UNKNOWN)
+	    fprintf(errstd, "Warning: Unable to determine video adapter in use in the present system.\n");
+	if (loader == &Third && adapter == VIDEO_CGA)
+	    fprintf(errstd, "Warning: Video adapter (CGA) is incompatible with the boot loader selected for\n"
+	    			"  installation ('install = menu').\n");
+	if (loader == &Bitmap && adapter < VIDEO_MCGA && adapter > VIDEO_UNKNOWN)
+	    fprintf(errstd, "Warning: Video adapter (%s) is incompatible with the boot loader selected for\n"
+	    			"  installation ('install = bitmap').\n", adapter==VIDEO_CGA ? "CGA" :
+	    			adapter==VIDEO_MDA ? "MDA" : "unknown");
+    }
+    
     return loader;
 }
-#endif
 
 
 /* kludge:  'append="..."' may not contain keywords acted upon by 
@@ -423,7 +437,7 @@ static void retrieve_crc(long crcval[])
     if (pass) pw_get(pass,crcval,0);
     else pw_get(cfg_get_strg(cf_options,"password"),crcval,1);
 
-    if (verbose >= 1) {
+    if (verbose >= 2) {
         printf("Password found is");
         for (i=0; i<MAX_PW_CRC; i++) printf(" %08lX", crcval[i]);
         printf("\n");
@@ -458,7 +472,16 @@ static void open_bsect(char *boot_dev)
 	    die("Can't put the boot sector on logical partition 0x%04X",
 	      (int)st.st_dev);
 	fd = dev_open(&dev,boot_dev_nr = st.st_dev,O_RDWR);
+	boot_devnam = dev.name;
     }
+/* new code to get boot device code */
+{
+    GEOMETRY geo;
+    geo_open(&geo, boot_devnam, O_RDONLY);
+    bios_boot = geo.device;
+    geo_close(&geo);
+}
+    
     if (boot_dev_nr && !is_first(boot_dev_nr) && !nowarn)
 	fprintf(errstd,"Warning: %s is not on the first disk\n",boot_dev ?
 	  boot_dev : "current root");
@@ -539,7 +562,7 @@ void bsect_open(char *boot_dev,char *map_file,char *install,int delay,
         "56000\000";
     GEOMETRY geo;
     struct stat st;
-    int i, speed, bitmap, j;
+    int i, speed, bitmap, j, dataend;
     int m_fd=0,kt_fd,sectors;
     char *message,*colon,*serial,*walk,*this,*keytable,*scheme;
     MENUTABLE *menu;
@@ -559,16 +582,16 @@ printf("sizeof(DESCR_SECTORS) = %d\n", sizeof(DESCR_SECTORS));
 printf("MAX_IMAGES = %d\n", MAX_IMAGES);
 #endif
     image = image_base = i = 0;
+    memset(&menuparams, 0, sizeof(menuparams));
     if (stat(map_file,&st) >= 0 && !S_ISREG(st.st_mode))
 	die("Map %s is not a regular file.",map_file);
     open_bsect(boot_dev);
     part_verify(boot_dev_nr,1);
-    ireloc = part_nowrite(boot_dev);
     if (ireloc>1) {
     	die("Filesystem would be destroyed by LILO boot sector: %s", boot_dev);
     }
     else if (ireloc==1) {
-	if (!nowarn) printf("Warning: boot record relocation is necessary: %s\n", boot_dev);
+	if (!nowarn) fprintf(errstd,"Warning: boot record relocation beyond BPB is necessary: %s\n", boot_dev);
     }
 
 #ifdef EARLY_MAP
@@ -593,21 +616,6 @@ printf("MAX_IMAGES = %d\n", MAX_IMAGES);
 /*  if (install) */ {
 	timestamp = bsect.par_1.timestamp; /* preserve timestamp */
 
-#ifndef LCF_BUILTIN
-	if (verbose > 0) {
-	    printf("Merging with %s",install);
-	    show_link(install);
-	    printf("\n");
-	}
-	i_fd = geo_open(&geo,install,O_RDONLY);
-	if (read(i_fd, table, MAX_BOOT_SIZE) != MAX_BOOT_SIZE)
-	    die("read %s: %s",boot_dev ? boot_dev : dev.name,strerror(errno));
-	check_version ((BOOT_SECTOR*)table, STAGE_FIRST);
-
-	memcpy((char*)&bsect, table, MAX_BOOT_SIZE);
-	if (fstat(i_fd,&st) < 0)
-	    die("stat %s: %s",boot_dev ? boot_dev : dev.name,strerror(errno));
-#else
 /* determine which secondary loader to use */
 
 	loader = select_loader();
@@ -618,38 +626,22 @@ printf("MAX_IMAGES = %d\n", MAX_IMAGES);
 		"TEXT" );
 	}
 	memcpy(&bsect, First.data, MAX_BOOT_SIZE);
-#endif
+
 	bsect.par_1.timestamp = timestamp;
 	map_begin_section(); /* no access to the (not yet open) map file
 		required, because this map is built in memory */
-#if defined(LCF_UNIFY) || defined(LCF_BUILTIN)
-#ifndef LCF_BUILTIN
-	(void)map_insert_file (&geo,1,(st.st_size+SECTOR_SIZE-1)/SECTOR_SIZE-1);
-#else
-	(void)map_insert_data (loader->data, loader->size);
-#endif
-#else
-	map_add (&geo,1,(st.st_size+SECTOR_SIZE-1)/SECTOR_SIZE-1);
-#endif
+	here2 = map_insert_data (loader->data, loader->size);
+	memcpy(&param2,loader->data,sizeof(param2));
 	sectors = map_write((SECTOR_ADDR*)secondary_map, (SECTOR_SIZE-4)/sizeof(SECTOR_ADDR)-2, 1);
-	memcpy(secondary_map+SECTOR_SIZE-4, "LILO", 4);
+	memcpy(secondary_map+SECTOR_SIZE-4, EX_MAG_STRING, 4);
 
 	/* fill in full size of secondary boot loader in paragraphs */
-	bsect.par_1.dataend = (sectors + 5 + MAX_DESCR_SECS) * (SECTOR_SIZE/16);
+	/*bsect.par_1.*/dataend = (sectors + 5 + MAX_DESCR_SECS) * (SECTOR_SIZE/16);
 
 	if (verbose > 1)
 	    printf("Secondary loader: %d sector%s (0x%0X dataend).\n",sectors,sectors == 1 ?
-	      "" : "s", bsect.par_1.dataend*16);
-#ifndef LCF_BUILTIN
-	if (lseek(i_fd, SECTOR_SIZE, SEEK_SET)!=SECTOR_SIZE)
-	    die("lseek %s: %s",boot_dev ? boot_dev : dev.name,strerror(errno));
-	if (read(i_fd,table,SECTOR_SIZE) != SECTOR_SIZE)
-	    die("read(2) %s: %s",boot_dev ? boot_dev : dev.name,strerror(errno));
-	stage_flags = ((BOOT_SECTOR*)table) -> par_2.stage;
-	geo_close(&geo);
-#else
+	      "" : "s", /*bsect.par_1.*/dataend*16);
 	stage_flags = ((BOOT_SECTOR*)(loader->data)) -> par_2.stage;
-#endif
 	if ((stage_flags & 0xFF) != STAGE_SECOND)
 	    die("Ill-formed boot loader; no second stage section");
 
@@ -668,32 +660,35 @@ printf("MAX_IMAGES = %d\n", MAX_IMAGES);
     temp_register(temp_map);
 #endif
 
-#if 1
 	map_begin_section();
 	map_add_sector(secondary_map);
 	(void) map_write(&bsect.par_1.secondary,1,0);
-#endif
+	if (bios_boot == bios_map)
+	    bsect.par_1.prompt |= FLAG_MAP_ON_BOOT;
+	if (bios_map & 0x80)	/* hard disk? */
+	    bsect.par_1.map_serial_no = serial_no[bios_map - 0x80];
+	if (verbose>=2)
+	    printf("bios_boot = 0x%02X  bios_map = 0x%02X  map==boot = %d  map S/N: %08lX\n",
+		bios_boot, bios_map,
+		!!(bsect.par_1.prompt&FLAG_MAP_ON_BOOT),
+		bsect.par_1.map_serial_no);
     
+/* code to get creation time of map file */
+    if (stat(temp_map, &st) < 0) die("Cannot get map file status");
+    param2.map_stamp = bsect.par_1.map_stamp = st.st_mtime;
+    if (verbose>=4) printf("Map time stamp: %08X\n", (int)bsect.par_1.map_stamp);
 
     *(unsigned short *) &bsect.sector[BOOT_SIG_OFFSET] = BOOT_SIGNATURE;
     message = cfg_get_strg(cf_options,"message");
     scheme = cfg_get_strg(cf_options,"bitmap");
     if (message && scheme) die("'bitmap' and 'message' are mutually exclusive");
     bsect.par_1.msg_len = 0;
-#ifdef LCF_BUILTIN
     bitmap = (loader==&Bitmap);
-#else
-    bitmap = scheme ? 1 : 0;
-#endif
     if (bitmap) {
 	message = scheme;
 	if (!(stage_flags & STAGE_FLAG_BMP4)) {
-#ifndef LCF_BUILTIN
-	    die("Boot loader does not support 'bitmap='");
-#else
 	    if (!nowarn) fprintf(errstd,"Warning: Non-bitmap capable boot loader; 'bitmap=' ignored.\n");
 	    message = NULL;
-#endif
 	}
     }
     j = -1;
@@ -710,6 +705,8 @@ printf("MAX_IMAGES = %d\n", MAX_IMAGES);
 	i = sizeof(BITMAPFILEHEADER)+sizeof(BITMAPHEADER)+sizeof(RGB)*16+
 				sizeof(BITMAPLILOHEADER);
 	if (bitmap || st.st_size>i) {
+	    int bits;
+	    
 	    j = get_std_headers(m_fd, &fhv, &bmhv, &lhv);
 	    if (j<0) die("read %s: %s", message, strerror(errno));
 	    if (j==0 || j>2) { /* definitely a bitmap file */
@@ -721,8 +718,12 @@ printf("MAX_IMAGES = %d\n", MAX_IMAGES);
 	        }
 	        if (bmh->size == sizeof(BITMAPHEADER) &&
 	        	bmh->width==640 && bmh->height==480 && 
-	            	bmh->numBitPlanes * bmh->numBitsPerPlane == 4) {
+	            	((bits = bmh->numBitPlanes * bmh->numBitsPerPlane) == 4 ||
+	            	  bits == 8) ) {
 	            if (!bitmap) die("Message specifies a bitmap file");
+		    if (bits>4 && adapter<VIDEO_VESA && !nowarn)
+			fprintf(errstd,"Warning: Video adapter does not support VESA BIOS extensions needed for\n"
+				       "  display of 256 colors.  Boot loader will fall back to TEXT only operation.\n");
 	        }
 		else if (bitmap) die("Unsupported bitmap");
 	    } else if (bitmap) die("Not a bitmap file");
@@ -737,7 +738,7 @@ printf("MAX_IMAGES = %d\n", MAX_IMAGES);
 #else
 	map_insert_file (&geo,0,(st.st_size+SECTOR_SIZE-1)/SECTOR_SIZE);
 #endif
-	sectors = map_end_section(&bsect.par_1.msg,0);
+	sectors = map_end_section(&menuparams.msg,0);
 	if (verbose >= 2)
 	    printf("%s: %d sector%s.\n",bitmap?"Bitmap":"Message",
 	    		sectors,sectors == 1 ?  "" : "s");
@@ -791,7 +792,7 @@ printf("MAX_IMAGES = %d\n", MAX_IMAGES);
 	    delay = 20;
 	}
     }
-    bsect.par_1.prompt = cfg_get_flag(cf_options,"prompt") ? FLAG_PROMPT : 0;
+    bsect.par_1.prompt |= cfg_get_flag(cf_options,"prompt") ? FLAG_PROMPT : 0;
     if (cfg_get_flag(cf_options,"suppress-boot-time-BIOS-data")) {
     	fprintf(errstd,"CAUTION:  boot-time BIOS data will not be saved\n");
     	bsect.par_1.prompt |= FLAG_NOBD;
@@ -832,8 +833,12 @@ printf("MAX_IMAGES = %d\n", MAX_IMAGES);
 	    die("%s: bad keyboard translation table",keytable);
 	(void) close(kt_fd);
     }
+#if 0
     menu = (MENUTABLE*)&table[256];
     memset(menu, 0, 256);
+#else
+    menu = &menuparams;
+#endif
     memcpy(&(menu->row), &(lhv.row), sizeof(lhv) - sizeof(lhv.size) - sizeof(lhv.magic));
 
     if ((scheme = cfg_get_strg(cf_options,"menu-scheme"))) {
@@ -1149,7 +1154,9 @@ static void bsect_done(char *name,IMAGE_DESCR *descr)
     }
     if (verbose >= 3) {
 	printf("%4s<dev=0x%02x,hd=%d,cyl=%d,sct=%d>\n","",
-	  descr->start.device,descr->start.head,descr->start.track,
+	  descr->start.device,
+	  descr->start.head,
+	  descr->start.track,
 	  descr->start.sector);
 	if (*options) printf("%4s\"%s\"\n","",options);
     }
@@ -1195,69 +1202,48 @@ void check_fallback(void)
 
 void bsect_update(char *backup_file, int force_backup, int pass)
 {
-    struct stat st;
-    char temp_name[PATH_MAX+1];
-    int bck_file;
     BOOT_SECTOR bsect_wr;
+static int timestamp = 0;
 
-    if (!backup_file) {
-	sprintf(temp_name, BACKUP_DIR "/boot.%04X", boot_dev_nr);
-	backup_file = temp_name;
+    if (pass>=0 && !timestamp) {
+        timestamp = make_backup(backup_file, force_backup, &bsect_orig,
+    						boot_dev_nr, "boot sector");
+        if (timestamp) bsect.par_1.timestamp = timestamp;
     }
-    bck_file = open(backup_file, O_RDONLY);
-    if (bck_file >= 0 && force_backup) {
-	(void) close(bck_file);
-	bck_file = -1;
-    }
-    if (bck_file >= 0) {
-	if (verbose > 0)
-	    printf("%s exists - no backup copy made.\n",backup_file);
-    }
-    else {
-	if ((bck_file = creat(backup_file, 0644)) < 0)
-	    die("creat %s: %s",backup_file, strerror(errno));
-	if (write(bck_file, (char *)&bsect_orig, SECTOR_SIZE) != SECTOR_SIZE)
-	    die("write %s: %s", backup_file, strerror(errno));
-	if (verbose > 0)
-	    printf("Backup copy of boot sector in %s\n", backup_file);
-	if (fstat(bck_file, &st) < 0)
-	    die("fstat %s: %s",backup_file,strerror(errno));
-	bsect.par_1.timestamp = st.st_mtime;
-    }
-    if (close(bck_file) < 0) die("close %s: %s",backup_file,strerror(errno));
-    if (pass==0) {
-	MENUTABLE *menu = (void*)&table[256];
-#if 0	
-	map_begin_section();
-	map_add_sector(secondary_map);
-	(void) map_write(&bsect.par_1.secondary,1,0);
+
+#ifndef LCF_UNIFY
+# error "Bios Translation algorithms require '-DUNIFY' in Makefile"
 #endif
-/*	map_descrs(&descrs, bsect.par_1.descr, &bsect.par_1.dflcmd);  */
-	map_descrs(&descrs, menu->mt_descr, &bsect.par_1.dflcmd);
-
+    if (pass<1) {	/* BIOS_TT logic */
+	MENUTABLE *menu = &menuparams;
+	map_descrs(&descrs, menu->mt_descr, &menuparams.dflcmd);
+	menuparams.raid_dev_mask = raid_mask(menuparams.raid_offset);
+	memcpy(menuparams.serial_no, serial_no, sizeof(serial_no));
+	memcpy(table+256, &menuparams, sizeof(menuparams));
 	((long*)table)[SECTOR_SIZE/sizeof(long)-2] = crc32(table, SECTOR_SIZE-2*sizeof(long), CRC_POLY1);
 	map_begin_section();
 	map_add_sector(table);
 	(void) map_write(&bsect.par_1.keytab,1,0);
 
-	map_close();
-    }
+	map_close(&param2, here2);
+    }	/* if (pass<1) ...	*/
+
+if (pass>=0) {
     if (lseek(fd,0,0) < 0)
-	die("lseek %s: %s",boot_devnam ? boot_devnam : dev.name,
-	  strerror(errno));
-    if (verbose > 0) printf("Writing boot sector.\n");
+	die("lseek %s: %s",
+		boot_devnam ? boot_devnam : dev.name,
+		strerror(errno));
 
 #if 1
-/*    if ( MAJOR(boot_dev_nr)==MAJOR_FD  && */
     if (ireloc &&
     	  bsect.par_1.cli == 0xFA
-#if 0
-    	  	  &&  bsect.par_1.call_ins == 0xE8
-#endif
     	 						 ) {
 /* perform the relocation of the boot sector */
 	int len = bsect.par_1.code_length;
 	int space = BOOT_SIG_OFFSET - len;
+	
+	if (len==0) die ("First stage loader is not relocatable.");
+	
 	space &= 0xFFF0;	/* roll back to paragraph boundary */
         bsect_wr = bsect_orig;
 	memcpy(&bsect_wr.sector[space], &bsect, len);
@@ -1269,6 +1255,12 @@ void bsect_update(char *backup_file, int force_backup, int pass)
 	    bsect_wr.sector[0] = 0xE9;		/* jmp near */
 	    *(short*)&bsect_wr.sector[1] = space - 3;
 	}
+	if (bsect_wr.sector[space+1] == 0xEB)	{  /* jmp short */
+	    len = space>>4;
+	    space += (signed)bsect_wr.sector[space+2] + 3;
+	    if (bsect_wr.sector[space] == 0xB8)	/* mov ax,#07C0 */
+		*(short*)&bsect_wr.sector[space+1] += len;
+	}
 /***	bsect = bsect_orig;  ***/
 	if (verbose >= 1) printf("Boot sector relocation performed\n");
     }
@@ -1287,15 +1279,24 @@ void bsect_update(char *backup_file, int force_backup, int pass)
     	die("LILO internal error:  Would overwrite Partition Table");
  /* failsafe check */
     	
-    if (write(fd, (char *)&bsect_wr, SECTOR_SIZE) != SECTOR_SIZE)
+    if (!test && write(fd, (char *)&bsect_wr, SECTOR_SIZE) != SECTOR_SIZE)
 	die("write %s: %s",boot_devnam ? boot_devnam : dev.name,
 	  strerror(errno));
+
+} /* if (pass>=0) ... */
+
     if (!boot_devnam) dev_close(&dev);
     else if (close(fd) < 0) {
 	    unbootable();
 	    die("close %s: %s",boot_devnam,strerror(errno));
 	}
+
+
+#if 0
     if (pass==0) {
+#else
+    if (pass<1) {
+#endif
 	pw_file_update(passw);
 	temp_unregister(temp_map);
 	if (rename(temp_map,map_name) < 0) {
@@ -1312,7 +1313,7 @@ void bsect_cancel(void)
 #if 0
     map_descrs(&descrs, bsect.par_1.descr, &bsect.par_1.dflcmd);
 #endif
-    map_close();
+    map_close(NULL,0);
     if (boot_devnam) (void) close(fd);
     else dev_close(&dev);
     temp_unregister(temp_map);
@@ -1436,20 +1437,32 @@ void bsect_uninstall(char *boot_dev,char *backup_file,int validate)
 
 
 void bsect_raid_update(char *boot_dev, unsigned long raid_offset, 
-	char *backup_file, int force_backup, int pass)
+	char *backup_file, int force_backup, int pass, int mask)
 {
     BOOT_SECTOR bsect_save;
+    int prompt = bsect.par_1.prompt;
 
-    if (pass > 0) {    
+    if (pass<0) bsect_update(backup_file, force_backup, pass);
+
+    if (pass != 0) {    
 	bsect_save = bsect;			/* save the generated boot sector */
         open_bsect(boot_dev);
         memcpy(&bsect, &bsect_save, MAX_BOOT_SIZE);	/* update the subject boot sector */
         bsect.par_1.raid_offset = raid_offset;	/* put in the new partition offset */
+#if 0
         bsect.par_1.prompt &= FLAG_SAVE;	/* clear all RAID flags */
+#else
+        bsect.par_1.prompt &= mask;		/* clear all RAID flags */
+#endif
         bsect.par_1.prompt |= raid_flags;	/* update the raid flags */
         *(unsigned short *) &bsect.sector[BOOT_SIG_OFFSET] = BOOT_SIGNATURE;
     }
+    
+    if (pass<0) pass = -pass;
+    
     bsect_update(backup_file, force_backup, pass);
+
+    bsect.par_1.prompt = prompt;	/* restore the flag byte */
 }
 
 
