@@ -1,7 +1,7 @@
 /* cfg.c  -  Configuration file parser */
 /*
 Copyright 1992-1997 Werner Almesberger.
-Copyright 1999-2001 John Coffman.
+Copyright 1999-2003 John Coffman.
 All rights reserved.
 
 Licensed under the terms contained in the file 'COPYING' in the 
@@ -20,6 +20,7 @@ source directory.
 #include <ctype.h>
 #include <string.h>
 
+#include "lilo.h"
 #include "common.h"
 #include "temp.h"
 #include "cfg.h"
@@ -141,6 +142,7 @@ CONFIG cf_other[] = {
   { cft_strg, "loader",		NULL,		NULL,NULL },
   { cft_strg, "table",		NULL,		NULL,NULL },
   { cft_flag, "unsafe",		NULL,		NULL,NULL },
+  { cft_flag, "master-boot",	NULL,		NULL,NULL },
   { cft_strg, "map-drive",	do_map_drive,	NULL,NULL },
   { cft_flag, "change",		do_change,	NULL,NULL },
   { cft_link, NULL,		&cf_all,	NULL,NULL }};
