@@ -2,7 +2,7 @@
 /* common.h  -  Common data structures and functions. */
 /*
 Copyright 1992-1998 Werner Almesberger.
-Copyright 1999-2002 John Coffman.
+Copyright 1999-2003 John Coffman.
 All rights reserved.
 
 Licensed under the terms contained in the file 'COPYING' in the 
@@ -15,14 +15,22 @@ source directory.
 #define COMMON_H
 
 #ifndef LILO_ASM
+#include <fcntl.h>
 #include <sys/stat.h>
 #include <asm/types.h>
-#include <linux/genhd.h>
+/*#include <linux/genhd.h>*/
 
-#include "lilo.h"
+/*#include "lilo.h"*/
 
+#ifdef O_ACCMODE
+# define O_NOACCESS O_ACCMODE
+#else
+/* open a file for "no access" */
+# define O_NOACCESS 3
+#endif
 
-#define O_NOACCESS 3  /* open a file for "no access" */
+/* special for LILO, bypass the actual open in  dev_open( , ,-1)   */
+#define O_BYPASS -1
 
 #endif
 
@@ -64,6 +72,54 @@ source directory.
 
 
 
+;*/typedef struct {			/*
+							block	0
+;*/    unsigned char  bytes_per_sector[2];	/*
+						bpb_bytes_per_sector:	.blkb	2
+;*/    unsigned char  sectors_per_cluster;	/*
+						bpb_sectors_per_cluster:.blkb	1
+;*/    unsigned char  reserved_sectors[2];	/*
+								.blkb	2
+;*/    unsigned char  number_of_FATs;		/*
+								.blkb	1
+;*/    unsigned char  root_dir_entries[2];	/*
+								.blkb	2
+;*/    unsigned char  total_sectors[2];		/*
+							.blkb	2
+;*/    unsigned char  media_descriptor;		/*
+							.blkb	1
+;*/    unsigned char  sectors_per_FAT[2];	/*
+							.blkb	2
+;*/    unsigned char  sectors_per_track[2];	/* DOS v.3
+							.blkb	2
+;*/    unsigned char  heads_per_cylinder[2];	/* DOS v.3
+							.blkb	2
+;*/    unsigned char  hidden_sectors[4];	/*
+							.blkb	4
+;*/    unsigned char  total_sectors_long[4];	/* if total_sectors==0
+							.blkb	4
+;*/    unsigned char  reserved[7];		/* pad to 32 bytes
+							.blkb	7
+;*/} BIOS_PARAMETER_BLOCK;	/*
+						bpb_size:
+							endb
+							
+							
+							
+;*/typedef struct {		/*
+						block	0
+;*/    unsigned char jump[3];	/* jump to boot code
+						bpdos_jump:	.blkb	3
+;*/    char system[8];		/* system ID
+						bpdos_system:	.blkb	8
+;*/    BIOS_PARAMETER_BLOCK bpb;	/* BIOS parameter block
+						bpdos_bpb:	.blkb	bpb_size
+;*/} BOOT_PARAMS_DOS;	/* DOS fat header
+						bpdos_size:
+							endb
+							
+							
+
 ;*/typedef struct {					/*
 							block	0
 ;*/    unsigned char cli;     /* clear interrupt flag instruction
@@ -85,31 +141,49 @@ source directory.
 						par1_raid_offset: .blkb	4
 ;*/    unsigned long timestamp; /* timestamp for restoration
 						par1_timestamp:	.blkb	4
-;*/    unsigned short timeout; /* 54 msec delay until input time-out,
-			       0xffff: never 
+;*/    unsigned long map_stamp; /* timestamp for this installation (map creation)
+						par1_mapstamp:	.blkb	4
+;*/    unsigned long map_serial_no; /* volume serial no. / id containing the map file
+						par1_map_serial_no:	.blkb	4
+;*/    unsigned short timeout; /* 54 msec delay until input time-out, 0xffff: never 
 						par1_timeout:	.blkb	2
 ;*/    unsigned short delay; /* delay: wait that many 54 msec units.
 						par1_delay:	.blkb	2
-;*/    unsigned short dataend; /* full size of second-stage, including parameter line
-						par1_dataend:	.blkb	2
-;*/    unsigned short msg_len; /* 0 if none
-						par1_msg_len:	.blkb	2
-;*/    SECTOR_ADDR msg; /* initial greeting message
-						par1_msg:	.blkb	sa_size
-;*/    SECTOR_ADDR dflcmd; /* default command line
-						par1_dflcmd:	.blkb	sa_size
-;*/    unsigned char prompt; /* FLAG_PROMPT=always, FLAG_RAID install
-						par1_prompt:	.blkb	1
 ;*/    SECTOR_ADDR keytab; /* keyboard translation table
 						par1_keytab:	.blkb	sa_size
 ;*/    SECTOR_ADDR secondary; /* sectors of the second stage loader
 						par1_secondary:	.blkb	sa_size
+;*/    unsigned short msg_len; /* 0 if none
+						par1_msg_len:	.blkb	2
+;*/    unsigned char prompt; /* FLAG_PROMPT=always, FLAG_RAID install
+						par1_prompt:	.blkb	1
 ;*/} BOOT_PARAMS_1; /* first stage boot loader 
 								.align	4
 						par1_size:
 							endb
 
 
+
+;*/typedef struct {	/* second stage parameters
+							block	0
+;*/    char jump[6]; /* jump over the data
+						par2_jump:	.blkb	6
+;*/    char signature[4]; /* "LILO"
+						par2_signature:	.blkb	4
+;*/    unsigned short stage,version;	/*
+						par2_stage:	.blkb	2
+						par2_version:	.blkb	2
+;*/    short unused;			/*
+						par2_unused:	.blkb	2
+;*/    unsigned long map_stamp;		/* time of creation of the map file
+						par2_mapstamp:	.blkb	4
+;*/} BOOT_PARAMS_2; /* second stage boot loader
+								.align	4
+						par2_size:
+							endb
+
+
+
 ;*/typedef struct {					/*
 							block	0
 ;*/	char menu_sig[4];	/* "MENU" or "BMP4" signature, or NULs if not present
@@ -151,16 +225,27 @@ source directory.
 ;*/	short mincol, reserved[3];	/* BMP min per col before spill to next, reserved spacer
 						mt_mincol:	.blkw	1
 								.blkw	3
-;*/	char unused[160-MAX_DESCR_SECTORS*sizeof(SECTOR_ADDR)];		/* spacer to Descriptor ptrs
-						mt_unused:	.blkb	160-sa_size*MAX_DESCR_SECTORS_asm
-;*/	SECTOR_ADDR mt_descr[MAX_DESCR_SECTORS];	/* descriptor disk addresses
-						mt_descr:	.blkb	sa_size*MAX_DESCR_SECTORS_asm
+
+;*/	unsigned long serial_no[MAX_BIOS_DEVICES];	/* Known device serial nos. 0x80 .. 0x8F
+						mt_serial_no:	.blkw	MAX_BIOS_DEVICES_asm*2
+;*/	unsigned long raid_offset[MAX_RAID_DEVICES];	/* RAID offsets for flagged devices
+						mt_raid_offset:	.blkw	MAX_RAID_DEVICES_asm*2
+;*/	unsigned short raid_dev_mask;			/* 16 bit raid device mask flagging items in serial_no
+						mt_raid_dev_mask: .blkw	1
+;*/	SECTOR_ADDR msg; /* initial greeting message
+						mt_msg:	.blkb	sa_size
+;*/	SECTOR_ADDR dflcmd; /* default command line
+						mt_dflcmd:	.blkb	sa_size
+;*/	SECTOR_ADDR mt_descr[MAX_DESCR_SECS];	/* descriptor disk addresses
+						mt_descr:	.blkb	sa_size*MAX_DESCR_SECS_asm
+;*/	char unused[150-MAX_BIOS_DEVICES*sizeof(long)-(MAX_RAID_DEVICES)*sizeof(long)-MAX_DESCR_SECS*sizeof(SECTOR_ADDR)];		/* spacer
+						mt_unused:	.blkb	150-sa_size*MAX_DESCR_SECS_asm-4*MAX_BIOS_DEVICES_asm-4*MAX_RAID_DEVICES_asm
 ;*/	short checksum[2];		/* checksum longword
 						mt_cksum:	.blkw	2
 ;*/	unsigned char mt_flag;		/* contains the FLAG_NOBD only
 						mt_flag:	.blkb	1
-;*/	char unused2[3];		/* spacer beyond checksum
-						mt_unused2:	.blkb	3
+;*/	char unused2;			/* spacer beyond checksum
+						mt_unused2:	.blkb	1
 ;*/} MENUTABLE;		/* MENU and BITMAP parameters at KEYTABLE+256
 						mt_size:
 							endb
@@ -168,11 +253,13 @@ source directory.
 ;*/
 
 #ifndef LILO_ASM
+#if 0
 typedef struct {
     char jump[6]; /* jump over the data */
     char signature[4]; /* "LILO" */
     unsigned short stage,version;
 } BOOT_PARAMS_2; /* second stage boot loader */
+#endif
 
 typedef struct {
     char jump[6]; /* jump over the data */
@@ -185,27 +272,6 @@ typedef struct {
     unsigned char ptable[PARTITION_ENTRY*PARTITION_ENTRIES]; /* part. table */
 } BOOT_PARAMS_C; /* chain loader */
 
-typedef struct {
-    unsigned char  bytes_per_sector[2];
-    unsigned char  sectors_per_cluster;
-    unsigned char  reserved_sectors[2];
-    unsigned char  number_of_FATs;
-    unsigned char  root_dir_entries[2];
-    unsigned char  total_sectors[2];
-    unsigned char  media_descriptor;
-    unsigned char  sectors_per_FAT[2];
-    unsigned char  sectors_per_track[2];	/* DOS v.3 */
-    unsigned char  heads_per_cylinder[2];	/* DOS v.3 */
-    unsigned char  hidden_sectors[4];
-    unsigned char  total_sectors_long[4];	/* if total_sectors==0 */
-    unsigned char  reserved[7];		/* pad to 32 bytes */
-} BIOS_PARAMETER_BLOCK;
-
-typedef struct {
-    unsigned char jump[3];	/* jump to boot code */
-    char system[8];		/* system ID */
-    BIOS_PARAMETER_BLOCK bpb;	/* BIOS parameter block */
-} BOOT_PARAMS_DOS;	/* DOS fat header */
 
 typedef union {
     BOOT_PARAMS_1 par_1;
@@ -259,15 +325,18 @@ typedef struct {
 #define alloc_t(t) ((t *) alloc(sizeof(t)))
 
 typedef enum {X_NULL=0, X_NONE, X_AUTO, X_MBR_ONLY, X_SPEC} EXTRA;
+typedef enum {AD_ANY=0, AD_GEOMETRIC, AD_LINEAR, AD_LBA32=4} ADDR_MODE;
 
 extern EXTRA extra;
 extern int verbose,test,compact,linear,nowarn,lba32,autoauto,passw,geometric;
+extern int bios_boot, bios_map;
 extern int ireloc, force_fs;
-extern int boot_dev_nr,raid_flags,do_md_install,zflag,eflag;
+extern int boot_dev_nr,raid_flags,do_md_install,ndisk,zflag,eflag;
 extern unsigned short drv_map[DRVMAP_SIZE+1]; /* needed for fixup maps */
 extern int curr_drv_map;
 extern unsigned long prt_map[PRTMAP_SIZE+1];
 extern int curr_prt_map, config_read;
+extern unsigned long serial_no[MAX_BIOS_DEVICES];
 #if 0
 extern unsigned long crc_polynomial[MAX_PW_CRC];
 #endif
@@ -317,4 +386,3 @@ void show_link(char *name);
 
 #endif
 #endif
-
