@@ -1,7 +1,7 @@
 /* geometry.c  -  Device and file geometry computation */
 /*
 Copyright 1992-1998 Werner Almesberger.
-Copyright 1999-2001 John Coffman.
+Copyright 1999-2003 John Coffman.
 All rights reserved.
 
 Licensed under the terms contained in the file 'COPYING' in the 
@@ -23,9 +23,7 @@ source directory.
 #define	_I386_STATFS_H	/* two versions of statfs is not good ... */
 #endif
 #endif
-#include <linux/fs.h>
-#include <linux/hdreg.h>
-#include <linux/fd.h>
+
 #include <string.h>
 
 #include "config.h"
@@ -173,16 +171,17 @@ int has_partitions(dev_t dev)
       major == MAJOR_IDE3 || major == MAJOR_IDE4 ||
       major == MAJOR_IDE5 || major == MAJOR_IDE6 ||
       (major >= MAJOR_IDE7 && major <= MAJOR_IDE10) ||
-      major == MAJOR_XT || major == MAJOR_ESDI
-      	) return 0xFFC0;
+      major == MAJOR_XT || major == MAJOR_ESDI || major == MAJOR_ACORN
+      	) return 0xFFC0;	/* 6 bit partition mask */
       
     if (
       major == MAJOR_SD || (major >= MAJOR_SD2 && major <= MAJOR_SD8) ||
       major == MAJOR_AMI_HYP || major == MAJOR_HPT370 ||
-      major == MAJOR_FL || major == MAJOR_NFTL ||
+      (major >= MAJOR_EXPR && major <= MAJOR_EXPR+3) ||
       (major >= MAJOR_I2O && major <= MAJOR_I2O+7) ||
       (major >= MAJOR_SMART2 && major <= MAJOR_SMART2+7) ||
-      (major >= MAJOR_CISS && major <= MAJOR_CISS+7)
+      (major >= MAJOR_CISS && major <= MAJOR_CISS+7) ||
+      major == MAJOR_FTL || major == MAJOR_NFTL
         ) return 0xFFF0;	/* 4 bit partition mask */
 
     if ( major == MAJOR_IBM_iSER ||
@@ -446,25 +445,18 @@ static void geo_query_dev(GEOMETRY *geo,int device,int all)
 	    geo->start = 0;
 	    break;
 	case MAJOR_HD:
-	    /* fall through */
 	case MAJOR_IDE2:
-	    /* fall through */
 	case MAJOR_IDE3:
-	    /* fall through */
 	case MAJOR_IDE4:
-	    /* fall through */
 	case MAJOR_IDE5:
-	    /* fall through */
 	case MAJOR_IDE6:
 	case MAJOR_IDE7:
 	case MAJOR_IDE8:
 	case MAJOR_IDE9:
 	case MAJOR_IDE10:
-	    /* fall through */
 	case MAJOR_ESDI:
-	    /* fall through */
 	case MAJOR_XT:
-	    /* fall through */
+	case MAJOR_ACORN:
 	    if (ioctl(fd,HDIO_GETGEO,&hdprm) < 0)
 		die("geo_query_dev HDIO_GETGEO (dev 0x%04x): %s",device,
 		  strerror(errno));
@@ -521,7 +513,11 @@ static void geo_query_dev(GEOMETRY *geo,int device,int all)
 	    break;
 	case MAJOR_AMI_HYP:
 	case MAJOR_HPT370:
-	case MAJOR_FL:
+	case MAJOR_EXPR:
+	case MAJOR_EXPR+1:
+	case MAJOR_EXPR+2:
+	case MAJOR_EXPR+3:
+	case MAJOR_FTL:
 	case MAJOR_NFTL:
 	case MAJOR_SMART2+0:
 	case MAJOR_SMART2+1:
@@ -583,27 +579,23 @@ int is_first(int device)
     switch (MAJOR(device)) {
 	case MAJOR_FD:
 	    return !(device & 3);
+
 	case MAJOR_HD:
 	    return !(MINOR(device) >> 6);
+
 	case MAJOR_IDE2:
-	    /* fall through */
 	case MAJOR_IDE3:
-	    /* fall through */
 	case MAJOR_IDE4:
-	    /* fall through */
 	case MAJOR_IDE5:
-	    /* fall through */
 	case MAJOR_IDE6:
 	case MAJOR_IDE7:
 	case MAJOR_IDE8:
 	case MAJOR_IDE9:
 	case MAJOR_IDE10:
-	    /* fall through */
 	case MAJOR_ESDI:
-	    /* fall through */
 	case MAJOR_XT:
-	    /* fall through */
 	    return MINOR(device) >> 6 ? 0 : !last_dev(MAJOR_HD,64);
+
 	case MAJOR_SD:
 	case MAJOR_SD2:
 	case MAJOR_SD3:
@@ -641,6 +633,7 @@ int is_first(int device)
 	case MAJOR_I2O+6:
 	case MAJOR_I2O+7:
 	    return MINOR(device) >> 4 ? 0 : !last_dev(MAJOR_HD,64);
+
 	case MAJOR_DAC960:
 	case MAJOR_DAC960+1:
 	case MAJOR_DAC960+2:
@@ -651,6 +644,7 @@ int is_first(int device)
 	case MAJOR_DAC960+7:
 	case MAJOR_IBM_iSER:
 	    return MINOR(device) >> 3 ? 0 : !last_dev(MAJOR_HD,64);
+
 	default:
 	    return 1; /* user knows what (s)he's doing ... I hope */
     }
@@ -660,7 +654,7 @@ int is_first(int device)
 void geo_get(GEOMETRY *geo,int device,int user_device,int all)
 {
     DT_ENTRY *walk;
-    int inherited,keep_cyls;
+    int inherited,keep_cyls,is_raid=0;
 
 #ifdef LCF_LVM
     /*
@@ -733,6 +727,7 @@ void geo_get(GEOMETRY *geo,int device,int user_device,int all)
 	    if (ioctl(md_fd,GET_DISK_INFO,&md_disk_info) < 0)
 	        die("GET_DISK_INFO: %s", mdxxx);
 	    if (!(md_disk_info.state & (1 << MD_DISK_FAULTY))) {
+		is_raid = (device==boot_dev_nr);
 	        device = (md_disk_info.major << 8) | md_disk_info.minor;
 		break;
 	    }
@@ -801,6 +796,10 @@ void geo_get(GEOMETRY *geo,int device,int user_device,int all)
 	printf("%15s%d sectors. Partition offset: %d sectors.\n","",
 	  geo->sectors,geo->start);
     }
+    geo->raid = is_raid;
+
+/* make the serial number association */
+    if (!is_raid) register_bios(geo->device, device);
 }
 
 
@@ -823,6 +822,14 @@ int geo_open(GEOMETRY *geo,char *name,int flags)
     if (!S_ISREG(st.st_mode) && !S_ISBLK(st.st_mode))
 	die("%s: neither a reg. file nor a block dev.",name);
     geo->dev = S_ISREG(st.st_mode) ? st.st_dev : st.st_rdev;
+#if VERSION_MINOR>=50
+	if (verbose>=4) {
+		printf("geo_open: (%s) st_dev(file)=%04X  st_rdev(blk)=%04X\n",
+		name,
+		(int)st.st_dev,
+		(int)st.st_rdev );
+	}
+#endif
     geo_get(geo, geo->dev, user_dev, 1);
     geo->file = S_ISREG(st.st_mode) ? st.st_dev : 0;
     geo->boot = 0;
@@ -867,6 +874,7 @@ int geo_open_boot(GEOMETRY *geo,char *name)
 #endif
     geo_get(geo, geo->dev, -1, 0);
     geo->file = S_ISREG(st.st_mode);
+    geo->raid = 0;
     geo->boot = 1;
     geo->spb = 1;
     return geo->fd;
@@ -876,6 +884,7 @@ int geo_open_boot(GEOMETRY *geo,char *name)
 void geo_close(GEOMETRY *geo)
 {
     if (geo->fd) (void) close(geo->fd);
+    geo->fd = 0;
 }
 
 
@@ -889,8 +898,10 @@ int geo_comp_addr(GEOMETRY *geo,int offset,SECTOR_ADDR *addr)
     int block,sector;
     static int warnings = 0;
 
+#if 0
     if (linear && lba32)
        die("'linear' and 'lba32' (-l and -L) are mutually exclusive.");
+#endif
     if (geo->boot && offset >= SECTOR_SIZE)
 	die("Internal error: sector > 0 after geo_open_boot");
     block = offset/geo->spb/SECTOR_SIZE;
@@ -945,10 +956,15 @@ int geo_comp_addr(GEOMETRY *geo,int offset,SECTOR_ADDR *addr)
     sector = block*geo->spb+((offset/SECTOR_SIZE) % geo->spb);
     sector += geo->start;
 #endif
- /*   Always use CHS addressing on floppies:     JRC   */
-    if ((geo->device & 0x80) && (linear || lba32)) {
+ /*   DON'T always use CHS addressing on floppies:     JRC   */
+/*    if ((geo->device & 0x80) && (linear || lba32)) {	*/
+    if ((linear || lba32)) {
         addr->device = geo->device | (linear ? LINEAR_FLAG : (LBA32_FLAG|LBA32_NOCOUNT))
+#if 0
        		| (do_md_install && geo->file==boot_dev_nr ? RAID_REL_FLAG : 0);
+#else
+       		| (do_md_install && geo->raid ? RAID_REL_FLAG : 0);
+#endif
         addr->num_sect = linear ? 1 : (sector >> 24);
 	addr->sector = sector & 0xff;
 	addr->track = (sector >> 8) & 0xff;
