@@ -1,7 +1,7 @@
 /* lilo.c  -  LILO command-line parameter processing */
 /*
 Copyright 1992-1998 Werner Almesberger.
-Copyright 1999-2002 John Coffman.
+Copyright 1999-2003 John Coffman.
 All rights reserved.
 
 Licensed under the terms contained in the file 'COPYING' in the 
@@ -17,13 +17,15 @@ source directory.
 #include <ctype.h>
 #include <fcntl.h>
 #include <errno.h>
+#include <time.h>
+#include <limits.h>
 #include <sys/stat.h>
-
 #include <asm/page.h>
 
 #include "config.h"
-#include "common.h"
 #include "lilo.h"
+#include "common.h"
+#include "raid.h"
 #include "boot.h"
 #include "temp.h"
 #include "device.h"
@@ -34,489 +36,16 @@ source directory.
 #include "identify.h"
 #include "partition.h"
 #include "probe.h"
+#include "loader.h"
 #include "md-int.h"
 #include "edit.h"
 #include "flags.i"
 
-static int lowest;
-static md_array_info_t md_array_info;
-static DT_ENTRY *md_disk;
-static DT_ENTRY *disk;
-static unsigned long raid_base, raid_offset[MAX_RAID];
-static char *raid_mbr[MAX_RAID];
-static int raid_device[MAX_RAID+1];
-static int raid_bios[MAX_RAID+1];
-static int device, md_bios;
-enum {MD_NULL=0, MD_PARALLEL, MD_MIXED, MD_SKEWED};
-int do_md_install;
 char *config_file;		/* actual name of the config file */
 int config_read;		/* readable by other than root */
 FILE *errstd;
-static char *raid_list[MAX_RAID];
-static int list_index[MAX_RAID];
-static int ndisk, nlist, faulty;
 
 
-#define IS_COVERED    0x1000
-#define TO_BE_COVERED 0x2000
-#define COVERED   (IS_COVERED|TO_BE_COVERED)
-
-static int is_primary(int device)
-{
-    int mask;
-    
-    mask = has_partitions(device);
-    if (!mask) die("is_primary:  Not a valid device  0x%04X", device);
-    mask = device & ~mask;
-    return (mask && mask<=PART_MAX);
-}
-
-
-#if 0
-static int is_master(int device)
-{
-    int mask;
-    
-    mask = has_partitions(device);
-    if (!mask) die("is_master:  Not a valid device  0x%04X", device);
-    mask = device & ~mask;
-    return (!mask);
-}
-#endif
-
-
-static int master(int device)
-{
-    int mask;
-    
-    mask = has_partitions(device);
-    if (!mask) die("master:  Not a valid device  0x%04X", device);
-    return device & mask;
-}
-
-
-static int is_accessible(int device)
-{
-    int mask;
-    
-    mask = has_partitions(device);
-    if (!mask) die("is_accessible:  Not a valid device  0x%04X", device);
-    mask = device & ~mask;
-    return (mask<=PART_MAX);
-}
-
-
-static void raid_final(void)
-{
-    int pass, force;
-    char *cp;
-
-	if (verbose>=2)
-	printf("do_md_install: %s\n", do_md_install == MD_PARALLEL ? "MD_PARALLEL" :
-		do_md_install == MD_MIXED ? "MD_MIXED" :
-		do_md_install == MD_SKEWED ? "MD_SKEWED" : "unknown");
-		
-    if (extra == X_MBR_ONLY) {    
-        pass = 0;
-        while (pass < ndisk) {
-	    if (!test) {
-	        force = 0;
-	    
-  /* we need to re-visit the logic about backups */
-	        if ((cp=cfg_get_strg(cf_options,"force-backup"))) force=1;
-	        else cp=cfg_get_strg(cf_options,"backup");
-	    
-	        bsect_raid_update(raid_mbr[pass], raid_offset[pass],
-			cp, force, pass);
-	        if (!nowarn) fprintf(errstd, "The Master boot record of  %s  has been updated.\n", raid_mbr[pass]);
-
-	    } else {
-	        if (pass==0) {
-	    	    bsect_cancel();
-		    if (passw) fprintf(errstd,"The password crc file has *NOT* been updated.\n");
-		    fprintf(errstd, "The map file has *NOT* been altered.\n");
-	        }
-
-	        fprintf(errstd,"The Master boot record of  %s  has *NOT* been altered.\n", 
-	    		raid_mbr[pass]);
-	    }
-	    pass++;
-        }
-    } 
-    else {	/*  extra != X_MBR_ONLY   */
-    	char *boot = cfg_get_strg(cf_options,"boot");
-    	
-	raid_flags &= ~FLAG_RAID_DEFEAT;    /* change won't affect /dev/mdX */
-	if (!test) {
-	    force = 0;
-	    
-	    if ((cp=cfg_get_strg(cf_options,"force-backup"))) force=1;
-	    else cp=cfg_get_strg(cf_options,"backup");
-
-	/* write out the /dev/mdX boot records */	    
-	    bsect_raid_update(boot, 0L, cp, force, 0);
-	    if (!nowarn) fprintf(errstd, "The boot record of  %s  has been updated.\n", boot);
-	}
-	else {
-	    bsect_cancel();
-	    if (passw) fprintf(errstd,"The password crc file has *NOT* been updated.\n");
-	    fprintf(errstd, "The map file has *NOT* been updated.\n");
-	    fprintf(errstd,"The boot record of  %s  has *NOT* been updated.\n", 
-	    		boot);
-	}
-
-	if (extra == X_NONE || (extra == X_AUTO && do_md_install == MD_PARALLEL) ) return;
-
-	if (extra == X_SPEC)
-	for (pass = 0; pass < nlist; pass++) {
-	    int index;
-	    
-	    if (raid_bios[list_index[pass]] & 0xFF) {
-	    	index = list_index[pass];	/* within RAID set */
-	    }
-	    else {  /* not in the RAID set */
-	    	raid_flags |= FLAG_RAID_DEFEAT;  /* make outsider invisible */
-	    	index = lowest;
-	    }
-
-	    if (verbose>=2) printf("Specifed partition:  %s  raid offset = %08lX\n",
-					raid_list[pass], raid_offset[index]);
-
-	    if (!test) {
-	        bsect_raid_update(raid_list[pass], raid_offset[index],
-	    		NULL, 0, 1);
-	    }
-	    if (test || !nowarn)
-	        fprintf(errstd,"The boot record of  %s  has%s been updated.\n",
-			raid_list[pass], (test ? " *NOT*" : ""));
-
-	    raid_flags &= ~FLAG_RAID_DEFEAT; /* restore DEFEAT flag to 0 */
-	}
-	else {		/* extra = X_AUTO */
-	    for (pass = 0; pass < ndisk; pass++)
-	    if (!(raid_bios[pass] & IS_COVERED) && (raid_bios[pass] & 0xFF) != 0x80) {
-		if (!test)
-		    bsect_raid_update(raid_mbr[pass], raid_offset[pass],
-		    	NULL, 0, 1);
-		if (test || !nowarn)
-	            fprintf(errstd,"The Master boot record of  %s  has%s been updated.\n",
-			raid_mbr[pass], (test ? " *NOT*" : ""));
-	    }
-	}
-    }
-
-
-    if (raid_flags & FLAG_RAID_NOWRITE) {
-	if (!nowarn) {
-		fprintf(errstd, "Warning: FLAG_RAID_NOWRITE has been set.\n");
-		if (verbose >= 1)
-		fprintf(errstd,   "  The boot loader will be unable to update the stored command line;\n"
-		                  "  'lock' and 'fallback' are not operable; the 'lilo -R' boot command\n"
-		                  "  line will be locked.\n\n");
-	}
-    }
-
-}
-
-
-static long raid_setup(void)
-{
-    int pass, mask;
-    struct stat st;
-    int md_fd;
-    md_disk_info_t md_disk_info;
-    GEOMETRY geo;
-    char *boot, *extrap;
-    int ro_set, all_pri_eq, pri_index;
-    long pri_offset;
-    int raid_limit;
-    
-    if ((boot=cfg_get_strg(cf_options,"boot")) != NULL &&
-        strncmp("/dev/md",boot,7) == 0) {
-	if ((md_fd=open(boot,O_NOACCESS)) < 0)
-	    die("Unable to open %s",boot);
-	if (fstat(md_fd,&st) < 0)
-	    die("Unable to stat %s",boot);
-	if (!S_ISBLK(st.st_mode))
-	    die("%s is not a block device",boot);
-	if (ioctl(md_fd,GET_ARRAY_INFO,&md_array_info) < 0)
-	    die("Unable to get RAID info on %s",boot);
-	if ((md_array_info.major_version == 0) && (md_array_info.minor_version < 90))
-	    die("Raid versions < 0.90 are not supported");
-	if (md_array_info.level != 1)
-	    die("Only RAID1 devices are supported as boot devices");
-	if (!linear && !lba32) {
-#if 0
-	    cfg_set(cf_options,"lba32",NULL,NULL);
-#endif
-	    lba32 = 1;
-	    if (!nowarn)
-		fprintf(errstd,"Warning: RAID install requires LBA32 or LINEAR;"
-			" LBA32 assumed.\n");
-	}
-	extrap = cfg_get_strg(cf_options, RAID_EXTRA_BOOT);
-	extra = !extrap ? X_AUTO :
-		!strcasecmp(extrap,"none") ? X_NONE :
-		!strcasecmp(extrap,"auto") ? X_AUTO :
-		!strcasecmp(extrap,"mbr-only") ? X_MBR_ONLY :
-		    X_SPEC;
-	
-	do_md_install = MD_PARALLEL;
-
-	all_pri_eq = 1;
-	ro_set = pri_index = pri_offset = 0;
-	raid_flags = FLAG_RAID;
-	md_bios = 0xFF;			/* we want to find the minimum */
-	ndisk = 0;			/* count the number of disks on-line */
-	nlist = 0;
-	faulty = 0;
-	
-	device = (MD_MAJOR << 8) | md_array_info.md_minor;
-	
-    /* search the disk table for a definition */
-	md_disk = disktab;
-	while (md_disk && md_disk->device != device)
-	    md_disk = md_disk->next;
-	    
-	if (!md_disk) {
-	    md_disk = alloc_t(DT_ENTRY);
-	    md_disk->device = (MD_MAJOR << 8) | md_array_info.md_minor;
-	    md_disk->bios = -1;	/* use the default */
-	    md_disk->next = disktab;
-	    disktab = md_disk;
-	}
-
-	if (verbose >= 2) {
-	   printf("RAID info:  nr=%d, raid=%d, active=%d, working=%d, failed=%d, spare=%d\n",
-		md_array_info.nr_disks,
-		md_array_info.raid_disks,
-		md_array_info.active_disks,
-		md_array_info.working_disks,
-		md_array_info.failed_disks,
-		md_array_info.spare_disks );
-	}
-
-    /* scan through all the RAID devices */
-	raid_limit = md_array_info.raid_disks + md_array_info.spare_disks;
-   	for (pass=0; pass < raid_limit; pass++) {
-	    DEVICE dev;
-	    int disk_fd;
-	    char new_name[MAX_TOKEN+1];
-	    char *np;
-	    
-	    md_disk_info.number = pass;
-	    if (ioctl(md_fd,GET_DISK_INFO,&md_disk_info) < 0)
-		die("main: GET_DISK_INFO: %s", strerror(errno));
-	    device = (md_disk_info.major << 8) | md_disk_info.minor;
-            if(verbose>=2) printf("md: RAIDset device %d = 0x%04X\n", pass, device);	    
-	    if (device == 0) { /* empty slot left over from recovery process */
-	        faulty++;
-		continue;
-	    }
-	    disk_fd = dev_open(&dev,device,O_NOACCESS);
-	    if (md_disk_info.state & (1 << MD_DISK_FAULTY)) {
-		printf("disk %s marked as faulty, skipping\n",dev.name);
-		faulty++;
-		continue;
-	    }
-	    geo_get(&geo, device, -1, 1);
-	    disk = alloc_t(DT_ENTRY);
-	    if (verbose>=2)
-		printf("RAID scan: geo_get: returns geo->device = 0x%02X"
-		      " for device %04X\n", geo.device, device);
-	      
-	    disk->bios = geo.device;	/* will be overwritten */
-	    disk->device = device;
-	      /* used to mask above with 0xFFF0; forces MBR; sloppy, mask may be: 0xFFF8 */
-	    disk->sectors = geo.sectors;
-	    disk->heads = geo.heads;
-	    disk->cylinders = geo.cylinders;
-	    disk->start = geo.start;
-	    if (ndisk==0) raid_base = geo.start;
-	    raid_offset[ndisk] = geo.start - raid_base;
-	    raid_device[ndisk] = device;
-
-	    if (raid_offset[ndisk]) {
-	        do_md_install = MD_SKEWED;	 /* flag non-zero raid_offset */
-	    }
-
-	    if (all_pri_eq && is_primary(device)) {
-		if (ro_set) {
-		    all_pri_eq &= (pri_offset == raid_offset[ndisk]);
-		} else {
-		    pri_offset = raid_offset[ndisk];
-		    ro_set = 1;
-		    pri_index = ndisk;
-		}
-	    }
-
-	    if (geo.device < md_bios) {
-	        md_bios = geo.device;	/* find smallest device code, period */
-	        lowest = ndisk;		/* record where */
-	    }
-	    raid_bios[ndisk] = geo.device;  /* record device code */
-
-	    disk->next = disktab;
-	    disktab = disk;
-
-	    if (verbose >= 2 && do_md_install) {
-		printf("disk->start = %d\t\traid_offset = %ld (%08lX)\n",
-		   disk->start, (long)raid_offset[ndisk], (long)raid_offset[ndisk]);
-	    }
-   	
-	/* derive the MBR name, which may be needed later */
-	    strncpy(new_name,dev.name,MAX_TOKEN);
-	    new_name[MAX_TOKEN] = '\0';
-	    np = boot_mbr(dev.name, 0);
-	    if (!np) np = stralloc(new_name);
-	    raid_mbr[ndisk] = np;
-
-	    if (ndisk==0) {	/* use the first disk geometry */
-		md_disk->sectors = geo.sectors;
-		md_disk->heads = geo.heads;
-		md_disk->cylinders = geo.cylinders;
-		md_disk->start = geo.start;
-	    }
-	    
-	    ndisk++;  /* count the disk */
-   	}  /* for (pass=...    */
-
-   	if(close(md_fd) < 0) die("Error on close of %s", boot);
-   	raid_bios[ndisk] = 0;		/* mark the end */
-   	raid_device[ndisk] = 0;
-
-	all_pri_eq &= ro_set;
-	if (all_pri_eq && do_md_install == MD_SKEWED) {
-	    do_md_install = MD_MIXED;
-	}
-	else pri_index = lowest;
-
-	/* check that all devices have an accessible block for writeback info */
-	for (pass=0; pass < ndisk; pass++) {
-	    if (extra == X_MBR_ONLY)
-		raid_bios[pass] |= TO_BE_COVERED;
-
-	    if (extra == X_AUTO && raid_bios[pass] != 0x80) {
-		if (do_md_install == MD_SKEWED)  raid_bios[pass] |= TO_BE_COVERED;
-		if (do_md_install == MD_MIXED) {
-		    if (is_primary(raid_device[pass])) raid_bios[pass] |= IS_COVERED;
-		    else  raid_bios[pass] |= TO_BE_COVERED;
-		}
-	    }
-		
-	    if ((do_md_install == MD_PARALLEL && is_accessible(raid_device[pass]))
-		|| (do_md_install == MD_MIXED && pri_offset == raid_offset[pass]
-		        && is_primary(raid_device[pass]))
-		)    
-		raid_bios[pass] |= IS_COVERED;
-	}
-	   	
-	nlist = 0;
-	if (extra==X_SPEC) {
-	    char *next, *scan;
-	    
-	    scan = next = extrap;
-	    while (next && *next) {
-		scan = next;
-		while (isspace(*scan)) scan++;	/* deblank the line */
-		next = strchr(scan, ',');	/* find the separator */
-		if (next) *next++ = 0;		/* NUL terminate  scan */
-		    
-		if ((md_fd=open(scan,O_NOACCESS)) < 0)
-		    die("Unable to open %s", scan);
-		if (fstat(md_fd,&st) < 0)
-		    die("Unable to stat %s",scan);
-		if (!S_ISBLK(st.st_mode))
-		    die("%s is not a block device",scan);
-	    	mask = has_partitions(st.st_rdev);
-	    	if (!mask) die("%s (%04X) not a block device", scan, (int)st.st_rdev);
-		if (verbose>=2) printf("RAID list: %s is device 0x%04X\n",
-				scan, (int)st.st_rdev);	    	
-		close(md_fd);
-		
-		list_index[nlist] = ndisk;  /* raid_bios==0 here */
-		for (pass=0; pass < ndisk; pass++) {
-		    if (master(st.st_rdev) == master(raid_device[pass])) {
-		    	list_index[nlist] = pass;
-		    	if (st.st_rdev == raid_device[pass])
-			    die("Cannot write to a partition within a RAID set:  %s", scan);
-		    	else if (is_accessible(st.st_rdev))
-		    	    raid_bios[pass] |= IS_COVERED;
-		    	break;
-		    }
-		}
-		if (list_index[nlist] == ndisk) {
-#if 0
-		    raid_flags |= FLAG_RAID_NOWRITE;  /* disk is outside RAID set */
-#endif
-		    if (!nowarn) printf("Warning: device outside of RAID set  %s  0x%04X\n", 
-		    				scan, (int)st.st_rdev);
-		}
-		raid_list[nlist++] = stralloc(scan);
-	    }
-	    
-	}
-	
-	   	
-    /* if the install is to MBRs, then change the boot= name */
-	if (extra == X_MBR_ONLY) {
-	    if (cfg_get_strg(cf_options,"boot")) cfg_unset(cf_options,"boot");
-	    cfg_set(cf_options, "boot", (boot=raid_mbr[0]), NULL);
-	}
-	else {	/* if skewed install, disable mdX boot records as 
-							source of writeback info */
-	    if (do_md_install == MD_SKEWED) raid_flags |= FLAG_RAID_DEFEAT | 
-	    	(extra == X_NONE ? FLAG_RAID_NOWRITE : 0);
-	}
-
-	mask = 1;
-	for (pass=0; pass < ndisk; pass++) {
-	    mask &= !!(raid_bios[pass] & COVERED);
-	}
-	if (!mask) {
-	    raid_flags |= FLAG_RAID_NOWRITE;
-	}
-
-	if (raid_flags & FLAG_RAID_NOWRITE) {
-	    if (!nowarn) {
-		fprintf(errstd, "\nWarning: FLAG_RAID_NOWRITE has been set.\n");
-#if 0
-		if (verbose >= 1)
-		fprintf(errstd,   "  The boot loader will be unable to update the stored command line;\n"
-		                  "  'lock' and 'fallback' are not operable; the 'lilo -R' boot command\n"
-		                  "  line will be locked.\n\n");
-#endif
-	    }
-	}
-
-    /* if the disk= bios= did not specify the bios, then this is the default */
-	if (md_disk->bios < 0) {
-	    md_disk->bios = md_bios;
-	}
-	if (md_disk->bios < 0x80 || md_disk->bios > DEV_MASK)
-	   die("Unusual RAID bios device code: 0x%02X", md_disk->bios);
-	disk = disktab;
-
-	for (pass=0; pass < ndisk; pass++) {
-	    disk->bios = md_disk->bios;	  /* all disks in the array are */
-	    disk = disk->next;		  /*  assigned the same bios code */
-	}
-
-	if (!nowarn) {
-	    fprintf(errstd,
-	       "Warning: using BIOS device code 0x%02X for RAID boot blocks\n",
-	    	                            md_disk->bios);
-	}
-	return raid_offset[pri_index];
-    }	/* IF (test for a raid installation */
-    else {	/* not raid at all */
-	if (cfg_get_strg(cf_options, RAID_EXTRA_BOOT))
-	    die("Not a RAID install, '" RAID_EXTRA_BOOT "=' not allowed");
-	return 0L;
-    }
-}  /* void raid_setup(void) */
-#undef COVERED
-
 static void show_other(int fd)
 {
     BOOT_SECTOR buf[SETUPSECS-1];
@@ -540,9 +69,14 @@ static void show_other(int fd)
     drvmap = ((unsigned char *) buf+buf[0].par_c.drvmap);
     prtmap = drvmap+2*(DRVMAP_SIZE+1);
     while (drvmap[0] && drvmap[1]) {
-	printf("    BIOS drive 0x%02x is mapped to 0x%02x\n",drvmap[0],
-	  drvmap[1]);
-	drvmap += 2;
+	if (drvmap[0]==0xFF && drvmap[1]==0xFF) {
+	    printf("    Master-Boot:  BIOS drive will always boot as 0x80\n");
+	    drvmap += 4;
+	} else {
+	    printf("    BIOS drive 0x%02X is mapped to 0x%02X\n",drvmap[0],
+						drvmap[1]);
+	    drvmap += 2;
+	}
     }
     /* fix VERY old bug -- offset of 0 in PT is okay */
     while (prtmap[0] /*** && prtmap[1] ***/ ) {
@@ -557,6 +91,7 @@ static void show_images(char *map_file)
 {
     DESCR_SECTORS descrs;
     BOOT_SECTOR boot;
+    MENUTABLE menu;
     GEOMETRY geo;
     SECTOR_ADDR addr[4];
     char buffer[SECTOR_SIZE];
@@ -568,9 +103,13 @@ static void show_images(char *map_file)
 
     fd = geo_open(&geo,map_file,O_RDONLY);
     if (read(fd,buffer,SECTOR_SIZE) != SECTOR_SIZE)
-	die("read %s: %s",map_file,strerror(errno));
-    if (read(fd,(char *) &descrs,sizeof(descrs)) != sizeof(descrs))
-	die("read %s: %s",map_file,strerror(errno));
+	die("read1 %s: %s",map_file,strerror(errno));
+    if (read(fd,(char*)&descrs,sizeof(descrs)) != sizeof(descrs))
+	die("read2 %s: %s",map_file,strerror(errno));
+    if (lseek(fd, - sizeof(menu), SEEK_END) <= 0)
+	die("lseek %s: %s",map_file,strerror(errno));
+    if (read(fd,(char*)&menu,sizeof(menu)) != sizeof(menu))
+	die("read3 %s: %s",map_file,strerror(errno));
     tlba32  = (descrs.d.descr[0].start.device & LBA32_FLAG) != 0;
     tlinear = !tlba32 && (descrs.d.descr[0].start.device & LINEAR_FLAG);
     if (tlinear != linear  ||  tlba32 != lba32) {
@@ -578,7 +117,7 @@ static void show_images(char *map_file)
 	       tlinear?"linear":tlba32?"lba32":"no linear/lba32");
         linear = tlinear;  lba32 = tlba32;
     }
-    if (verbose > 0) {
+    if (verbose) {
 	bsect_read(cfg_get_strg(cf_options,"boot"),&boot);
 #if 1
 	if (boot.par_1.cli != 0xFA) {	/* relocation happened */
@@ -594,6 +133,7 @@ static void show_images(char *map_file)
 		die("Cannot recognize boot sector.");
 	}
 #endif
+	printf("Installed:  %s\n", ctime((time_t*)&boot.par_1.map_stamp));
 	printf("Global settings:\n");
 	tsecs = (boot.par_1.delay*2197+3999)/4000;
 	printf("  Delay before booting: %d.%d seconds\n",tsecs/10,tsecs % 10);
@@ -611,6 +151,8 @@ static void show_images(char *map_file)
 		boot.par_1.prompt & FLAG_LARGEMEM ? "" : " NOT");
 	printf("  %sRAID installation\n",
 		boot.par_1.prompt & FLAG_RAID ? "" : "Non-");
+	printf("  Boot device %s be used for the Map file\n",
+		boot.par_1.prompt & FLAG_MAP_ON_BOOT ? "WILL" : "will not");
 	if (!boot.par_1.port) printf("  Serial line access is disabled\n");
 	else printf("  Boot prompt can be accessed from COM%d\n",
 	      boot.par_1.port);
@@ -622,6 +164,9 @@ static void show_images(char *map_file)
 	if (*(unsigned short *) buffer != DC_MAGIC || !buffer[2])
 	    printf("  No default boot command line\n");
 	else printf("  Default boot command line: \"%s\"\n",buffer+2);
+	if (verbose>=3) {
+	    printf("Serial numbers %08lX\n", menu.serial_no[0]);
+	}
 	printf("Images:\n");
     }
     for (image = 0; image < MAX_IMAGES; image++)
@@ -642,8 +187,9 @@ static void show_images(char *map_file)
 		   sector = (sector<<8)+descrs.d.descr[image].start.head;
 	           sector = (sector<<8)+descrs.d.descr[image].start.track;
 		   sector = (sector<<8)+descrs.d.descr[image].start.sector;
-		   printf(" <dev=0x%02x,linear=%d>",
-		     descrs.d.descr[image].start.device,
+		   printf(" <dev=0x%02x,%s=%d>",
+		     descrs.d.descr[image].start.device&DEV_MASK,
+		     descrs.d.descr[image].start.device&LBA32_FLAG ? "lba32" : "linear",
 		     sector);
 		}
 	        else { /*  CHS addressing */
@@ -807,14 +353,17 @@ int main(int argc,char **argv)
 static char *bitmap_file;
     int query,more,version,uninstall,validate,activate,instmbr,geom;
     struct stat st;
-    int fd;
+    int fd, temp;
     long raid_offset;
 
     errstd = stderr;
+#if VERSION_MINOR>=50
+    if (sizeof(MENUTABLE)!=256) die("MENUTABLE is not 256 bytes (common.h)");
+#endif
     config_file = DFL_CONFIG;
     act1 = act2 = tell_param = 
 	    reboot_arg = identify = ident_opt = new_root = uninst_dev = NULL;
-    lowest = do_md_install = zflag =
+    do_md_install = zflag =
 	    query = version = uninstall = validate = activate = instmbr = 0;
     verbose = -1;
     name = *argv;
@@ -823,7 +372,7 @@ static char *bitmap_file;
     while (argc && **++argv == '-') {
 	argc--;
       /* first those options with a mandatory parameter */
-      /* Notably absent are "RuUv" */
+      /* Notably absent are "RuUvw" */
 	if (strchr("AbBCdDEfiImMPrsSTx", ch=(*argv)[1])) {
 	    if ((*argv)[2]) param = (*argv)+2;
 	    else {
@@ -875,7 +424,7 @@ fprintf(errstd,"argc=%d, *argv=%s, ch=%c param=%s\n", argc, *argv, ch, param);
 		force_fs=1;
 		break;
 	    case 'g':
-		geometric |= 1;
+		geometric |= AD_GEOMETRIC;
 		break;
 	    case 'i':
 		cfg_set(cf_options,"install",param,NULL);
@@ -890,10 +439,10 @@ fprintf(errstd,"argc=%d, *argv=%s, ch=%c param=%s\n", argc, *argv, ch, param);
 		}
 		break;
 	    case 'l':
-		geometric |= 2;
+		geometric |= AD_LINEAR;
 		break;
 	    case 'L':
-		geometric |= 4;
+		geometric |= AD_LBA32;
 		break;
 	    case 'm':
 		cfg_set(cf_options,"map",param,NULL);
@@ -987,6 +536,7 @@ fprintf(errstd,"REBOOT=\"%s\"\n", reboot_arg);
 	    case 'w':
 		cfg_set(cf_options,"nowarn",NULL,NULL);
 		nowarn = 1;
+		if ( (*argv)[2] == '+' ) nowarn = -1;
 		break;
 	    case 'x':
 		if (!strcmp(param,"none"))
@@ -999,15 +549,19 @@ fprintf(errstd,"REBOOT=\"%s\"\n", reboot_arg);
 		    cfg_set(cf_options,RAID_EXTRA_BOOT,param,NULL);
 		break;
 	    case 'X':
-#ifndef PAR1_PARAMS
+#if VERSION_MINOR>=50
+{
 		printf(
-		"-DCODE_START_1=%d -DCODE_START_2=%d "
+		"-DBOOT_PARAMS_1=%d -DBOOT_PARAMS_2=%d  PATH_MAX=%d"
 		  "\n"
 		  ,
 		  sizeof(BOOT_PARAMS_1),
-		  sizeof(BOOT_PARAMS_2)
+		  sizeof(BOOT_PARAMS_2),
+		  PATH_MAX
 		     );
-#if 1
+}
+#endif
+#if VERSION_MINOR>=50
 {
 #include "bitmap.h"
 	RGB pal[16];
@@ -1028,9 +582,52 @@ fprintf(errstd,"REBOOT=\"%s\"\n", reboot_arg);
 		  );
 }
 #endif
+#define crc(a,b) (~crc32((a),(b),CRC_POLY1))
+#define brev(x)  \
+ { register unsigned short u1,u2; register unsigned long u=(x); u1=u; u2=u>>16; \
+ u1=(u1>>8)|(u1<<8); u2=(u2>>8)|(u2<<8); x=(unsigned long)u1<<16|u2; }
+#define cc(x) crc(x.data,x.size)
+#define comma (cc(Second)|cc(Third)|cc(Bitmap))
+{
+  unsigned long j;
+  if (verbose>=5) {
+    if (crc(Second.data,Second.size)) {
+	j = crc(Second.data,Second.size-4);
+	brev(j);
+	printf("2=0x%lx\n", j);
+    }
+    if (crc(Third.data,Third.size)) {
+	j = crc(Third.data,Third.size-4);
+	brev(j);
+	printf("3=0x%lx\n", j);
+    }
+    if (crc(Bitmap.data,Bitmap.size)) {
+	j = crc(Bitmap.data,Bitmap.size-4);
+	brev(j);
+	printf("B=0x%lx\n\n", j);
+    }
+  }
+}
+
+#if VERSION_MINOR>=50
+{
+	dev_t dev; int i;
+	struct {
+		short a[0x1b6/2];
+		long  b;
+		short c;
+		char pt[64];
+		short d;
+		} x;
+	dev = 0xFFF0;
+	i = dev;
+	printf("dev_t is %ssigned size=%d  i=%08X\n", i==0xFFF0 ? "un" : "", sizeof(dev_t), i);
+	printf("size of x = %d\n", sizeof(x));
+	printf("Size of MENUTABLE = %d\n", sizeof(MENUTABLE));
+}
 #endif
 		printf("\nCFLAGS = " CFLAGS "\n");
-		printf("\nMAX_IMAGES = %d\n", MAX_IMAGES);
+		printf("\nMAX_IMAGES = %d\n\n", MAX_IMAGES);
 		exit(0);
 	    case 'z':
 		zflag++;	/* force zero of MBR 8-byte area */
@@ -1043,7 +640,7 @@ fprintf(errstd,"REBOOT=\"%s\"\n", reboot_arg);
     if (!new_root) new_root = getenv("ROOT");
     if (new_root && *new_root) {
 	if (chroot(new_root) < 0) die("chroot %s: %s",new_root,strerror(errno));
-	if (chdir("/dev") < 0 && !nowarn)
+	if (chdir("/dev") < 0 && nowarn==0)
 	    fprintf(errstd, "Warning: root at %s has no /dev directory\n", new_root);
 	if (chdir("/") < 0) die("chdir /: %s",strerror(errno));
     }
@@ -1058,10 +655,10 @@ fprintf(errstd,"REBOOT=\"%s\"\n", reboot_arg);
 	    return 0;
 	}
 	printf(", Copyright (C) 1992-1998 Werner Almesberger\n"
-	       "Development beyond version 21 Copyright (C) 1999-2002 John Coffman\n"
+	       "Development beyond version 21 Copyright (C) 1999-2003 John Coffman\n"
 	       );
-        if (verbose>0) printf("Released %s and compiled at %s on %s.\n",
-	        VERSION_DATE, __TIME__, __DATE__);
+        if (verbose>0) printf("Released %s%s and compiled at %s on %s.\n",
+	        VERSION_DATE, comma ? "," : "", __TIME__, __DATE__);
 #if 0 && defined(LCF_DSECS)
 	printf("MAX_IMAGES = %d\n", MAX_IMAGES);
 #endif
@@ -1077,6 +674,10 @@ fprintf(errstd,"REBOOT=\"%s\"\n", reboot_arg);
 
     fd = cfg_open(config_file);
     more = cfg_parse(cf_options);
+    
+    temp = cfg_get_flag(cf_options,"nowarn");
+    if (nowarn < 0) nowarn = 0;
+    else nowarn = temp;
 
     if (verbose>=6) printf("main: cfg_parse returns %d\n", more);
     if (tell_param) probe_tell(tell_param);
@@ -1111,11 +712,16 @@ fprintf(errstd,"REBOOT=\"%s\"\n", reboot_arg);
     }
     preload_dev_cache();
     
+    if (verbose<0 && cfg_get_strg(cf_options,"verbose"))
+	verbose = to_number(cfg_get_strg(cf_options,"verbose"));
+    if (verbose<0) verbose = 0;
+    if (verbose) errstd = stdout;
+
     compact = cfg_get_flag(cf_options,"compact");
     geom = cfg_get_flag(cf_options,"geometric");
     linear = cfg_get_flag(cf_options,"linear");
     lba32  = cfg_get_flag(cf_options,"lba32");
-    nowarn = cfg_get_flag(cf_options,"nowarn");
+    
     if (geom+linear+lba32 > 1)
 	die("May specify only one of GEOMETRIC, LINEAR or LBA32");
     if (geometric) {
@@ -1123,32 +729,32 @@ fprintf(errstd,"REBOOT=\"%s\"\n", reboot_arg);
 	    fprintf(errstd,"Ignoring entry '%s'\n", geom ? "geometric" :
 	    	linear ? "linear" : "lba32");
 	geom = linear = lba32 = 0;
-	if (geometric==4) lba32 = 1;
-	else if (geometric==2) linear = 1;
-	else if (geometric==1) geom = 1;
+	if (geometric==AD_LBA32) lba32 = 1;
+	else if (geometric==AD_LINEAR) linear = 1;
+	else if (geometric==AD_GEOMETRIC) geom = 1;
     }    
 #ifdef LCF_LBA32
     if (geom+linear+lba32 == 0) {
-	if (!nowarn) fprintf(errstd,"Warning: LBA32 addressing assumed\n");
+	if (!nowarn && verbose) fprintf(errstd,"Warning: LBA32 addressing assumed\n");
 	lba32 = 1;
     }
 #endif
     if (linear && !nowarn) fprintf(errstd,
-    	"Warning:  LINEAR is deprecated in favor of LBA32:  LINEAR specifies 24-bit\n"
+    	"\nWarning:  LINEAR is deprecated in favor of LBA32:  LINEAR specifies 24-bit\n"
 	"  disk addresses below the 1024 cylinder limit; LBA32 specifies 32-bit disk\n"
 	"  addresses not subject to cylinder limits on systems with EDD bios extensions;\n"
-	"  use LINEAR only if you are aware of its limitations.\n");
-    if (verbose<0 && cfg_get_strg(cf_options,"verbose"))
-	verbose = to_number(cfg_get_strg(cf_options,"verbose"));
-    if (verbose<0) verbose = 0;
+	"  use LINEAR only if you are aware of its limitations.\n\n");
     
     if (identify) identify_image(identify,ident_opt);
 
 /* test for a RAID installation */
     raid_offset = raid_setup();
-    if (verbose >= 2)
-        printf("raid_setup returns offset = %08lX  ndisk = %d\n", raid_offset, ndisk);
-	    
+    if (verbose >= 2) {
+	printf("raid_setup returns offset = %08lX  ndisk = %d\n", raid_offset, ndisk);
+	dump_serial_nos();    
+    }
+
+
 	if (uninstall)
 	    bsect_uninstall(uninst_dev ? uninst_dev : cfg_get_strg(cf_options,
 	      "boot"),cfg_get_strg(cf_options,"backup"),validate);
@@ -1188,18 +794,27 @@ fprintf(errstd,"REBOOT=\"%s\"\n", reboot_arg);
 	    die("No images have been defined or default image doesn't exist.");
 	check_fallback();
 	
+	if (verbose>=2) dump_serial_nos();
 	if (do_md_install) raid_final();
-	else if (!test)
-	    if (cfg_get_strg(cf_options,"force-backup"))
-		bsect_update(cfg_get_strg(cf_options,"force-backup"),1,0);
+	else if (!test) {
+	    char *cp;
+	    
+	    if (verbose) printf("Writing boot sector.\n");
+
+	    cp = cfg_get_strg(cf_options,"force-backup");
+	    if (cp) bsect_update(cp,1,0);
 	    else bsect_update(cfg_get_strg(cf_options,"backup"),0,0);
+
+	} 
 	else {
 	    bsect_cancel();
 	    if (passw)
-	        fprintf(errstd,"The password crc file has *NOT* been updated.\n");
-	    fprintf(errstd,"The boot sector and the map file have *NOT* been "
+	        printf("The password crc file has *NOT* been updated.\n");
+
+	    printf("The boot sector and the map file have *NOT* been "
 	      "altered.\n");
 	}
+	if (verbose>=4) dump_serial_nos();
 
 	
     return 0;
