@@ -28,12 +28,29 @@ source directory.
  * definitions into asm/boot.h instead of linux/config.h
  */
 
+#if 0
+
 #ifdef HAS_BOOT_H
 #include <asm/boot.h>
 #else
 #include <linux/config.h>
 #endif
 
+#else
+
+/* Don't touch these, unless you really know what you're doing. */
+#define DEF_INITSEG	0x9000
+#define DEF_SYSSEG	0x1000
+#define DEF_SETUPSEG	0x9020
+#define DEF_SYSSIZE	0x7F00
+
+/* Internal svga startup constants */
+#define NORMAL_VGA	0xffff		/* 80x25 mode */
+#define EXTENDED_VGA	0xfffe		/* 80x50 mode */
+#define ASK_VGA		0xfffd		/* ask for it at bootup */
+
+#endif
+
 #define nelem(a) (sizeof(a)/sizeof((a)[0]))
 #define S2(x) #x
 #define S(x) S2(x)
@@ -47,15 +64,21 @@ source directory.
 #define MAJOR_SD	8 /* SCSI disks 0-15 */
 #define MAJOR_MD	9 /* multi-disk RAID sets */
 #define MAJOR_XT	13 /* XT-type hard disks */
+#define MAJOR_ACORN	21 /* Acorn MFM controller */
 #define MAJOR_IDE2	22 /* IDE on second interface */
 #define MAJOR_IDE3	33 /* IDE on third interface */
 #define MAJOR_IDE4	34 /* IDE on fourth interface */
 #define MAJOR_ESDI	36 /* PS/2 ESDI drives */
+#define MAJOR_FTL	44 /* Flash Transition Layer on Memory Technology Device */
+#define MAJOR_PP	45 /* Parallel Port IDE drive */
+#define MAJOR_PPFD	47 /* Parallel Port floppy drive */
 #define MAJOR_DAC960	48 /* First Mylex DAC960 PCI RAID controller */
 #define MAJOR_IDE5	56 /* IDE on fifth interface */
 #define MAJOR_IDE6	57 /* IDE on sixth interface */
 #define MAJOR_LVM	58 /* Logical Volume Manager block device */
+#define MAJOR_EXPR	60 /* Experimental devices 60..63 */
 #define MAJOR_FL	62 /* M-Systems Disk-On-Chip 2000 ***experimental*** */
+
 #define MAJOR_SD2	65 /* SCSI disks 16-31 */
 #define MAJOR_SD3	66 /* SCSI disks 32-47 */
 #define MAJOR_SD4	67 /* SCSI disks 48-63 */
@@ -69,6 +92,7 @@ source directory.
 #define MAJOR_IDE8	89 /* IDE on eighth interface */
 #define MAJOR_IDE9	90 /* IDE on ninth interface */
 #define MAJOR_IDE10	91 /* IDE on tenth interface */
+#define MAJOR_PPDD	92 /* PPDD encrypted disks - not supported */
 #define MAJOR_NFTL	93 /* NAND Flash Translation Layer (Disk-On-Chip) */
 #define MAJOR_AMI_HYP	101 /* AMI Hyper Disk RAID controller */
 #define MAJOR_CISS	104 /* First CCISS Major 104-111 */
@@ -76,7 +100,6 @@ source directory.
 #define MAJOR_HPT370	114 /* HPT370 controller */
 #define MAJOR_EVMS	117 /* Enterprise Volume Management System */
 
-#define MAX_RAID	30	/* max number of RAID disks in a set */
 #define MAX_TOKEN	255 /* max device Token length */
 #define MAX_IMAGE_NAME	15 /* maximum name length (w/o terminating NUL) */
 #define MAX_DESCR_SECTORS  12  /* upper limit on MAX_DESCR_SECS */
@@ -135,6 +158,13 @@ source directory.
 #define BOOT_SIGNATURE	0xaa55	/* boot signature */
 #define BOOT_SIG_OFFSET	510	/* boot signature offset */
 #define MAGIC_SERIAL	0xC9CF	/* LILO installed serial number */
+#define MAX_BIOS_DEVICES  16	/* max hard disk devices */
+#define MAX_RAID	30	/* max number of RAID disks in a set */
+#if 0
+#define MAX_RAID_DEVICES  6	/* max raid devices reported to second.S */
+#else
+#define MAX_RAID_DEVICES MAX_DESCR_SECTORS-MAX_DESCR_SECS+6	/* max raid devices reported to second.S */
+#endif
 
 #define PART_LINUX_MINIX  0x81	/* Linux/MINIX partition */
 #define PART_LINUX_SWAP	  0x82	/* Linux swap partition */
@@ -196,6 +226,7 @@ source directory.
 #define EX_MAG_L	0x494c	/* magic number at ES:SI, "LI" */
 #define EX_MAG_H	0x4f4c	/* magic number at ES:SI+2, "LO" */
 #define EX_MAG_HL	0x4f4c494c  /* "LILO" */
+#define EX_MAG_STRING	"LILO"	/* magic signature string as as string */
 
 #define BIOS_MAX_DEVS	2	/* BIOS devices (2 fd, 2 hd) */
 #define BIOS_MAX_HEADS  256	/* 8 bits head number; really 255 */
@@ -210,6 +241,7 @@ source directory.
 #define FLAG_RAID_NOWRITE 8	/* defeat RAID writeback of command line */
 #define FLAG_NOBD	16	/* defeat BIOS data collection at boot time */
 #define FLAG_LARGEMEM	32	/* BIOS has MoveExtMemBlk support for 386 */
+#define FLAG_MAP_ON_BOOT 64	/* map file is on the boot device */
 #define FLAG_SAVE (~(FLAG_RAID|FLAG_RAID_DEFEAT|FLAG_RAID_NOWRITE))  /* All but raid flags */
 
 /* these are the descriptor flags */
@@ -247,7 +279,7 @@ source directory.
 #define LFLAG_USE_HEAP	0x80
 
 #define PRTMAP_SIZE	32	/* number of partition type mappings */
-#define DRVMAP_SIZE	8	/* number of drive mappings */
+#define DRVMAP_SIZE	24	/* number of drive mappings */
 
 #define CRC_POLY1 0x04c11db7
 #define CRC_POLY2 0x23a55379
@@ -282,6 +314,9 @@ MAX_IMAGE_NAME_asm = MAX_IMAGE_NAME	! **
 MAX_PW_CRC_asm	= MAX_PW_CRC		! **
 SECTOR_SIZE_asm = SECTOR_SIZE		! **
 MAX_MENU_TITLE_asm = MAX_MENU_TITLE	! **
+MAX_BIOS_DEVICES_asm = MAX_BIOS_DEVICES	! **
+MAX_RAID_DEVICES_asm = MAX_RAID_DEVICES	! **
+
 
 MAPcode   = MAX_SECONDARY_asm*0x200	! size of secondary code
 MAP	  = MAPcode+0x200		! plus space for cmdline input
@@ -304,12 +339,7 @@ SETUP_STACKSIZE = 2048		! stacksize for kernel setup.S
 STACK	  = 2048		! amount of stack space to reserve
 PARMLINE  = DATAEND		! 0x2A00	! parameter line construction area off.
 
-#if 0
-SSDIFFseg = FIRSTSEG-STACKSEG
-SSDIFF	  = SSDIFFseg<<4     ! 0xA000	! used to address FIRSTSEG from SS
-#else
 SSDIFF	= 0
-#endif
 
 !SLA_SIZE  = 0x9E00		! setup load area size
 				! as the max. heap pointer in newer kernels
@@ -371,5 +401,23 @@ CL_RAMDISK_MAX	= CL_POINTER+4		! ramdisk_max; header version 0x0203
 
 #endif
 
+/* the following configuration variable are now  required
+   don't compile without them ...
+ */
+
+#ifndef LCF_M386
+# define LCF_M386
+#endif
+#ifndef LCF_VARSETUP
+# define LCF_VARSETUP
+#endif
+#ifndef LCF_UNIFY
+# define LCF_UNIFY
+#endif
+#ifndef LCF_BUILTIN
+# define LCF_BUILTIN
+#endif
+
+
 #endif
 
