@@ -85,6 +85,7 @@ void map_create(char *name)
     memset(buffer,0,SECTOR_SIZE);
     *(unsigned short *) buffer = DC_MGOFF;
     map_file = geo_open(&map_geo,name,O_RDWR);
+    bios_map = map_geo.device;	/* set device code of map file */
     if (do_md_install) {
 	struct stat st;
 	if(fstat(map_file,&st)) die("map_create: cannot fstat map file");
@@ -148,8 +149,14 @@ void map_descrs(DESCR_SECTORS *descr, SECTOR_ADDR* addr, SECTOR_ADDR* dflcmd)
 }
 
 
-void map_close(void)
+void map_close(BOOT_PARAMS_2 *param2, off_t here)
 {
+    if (param2) {
+	if (lseek(map_file, here, SEEK_SET) != here)
+	    die("map_close: lseek");
+	if (write(map_file,(void*)param2,sizeof(BOOT_PARAMS_2))!=sizeof(BOOT_PARAMS_2))
+	    die("map_close: write");
+    }
     geo_close(&map_geo);
 }
 
