@@ -12,8 +12,6 @@ source directory.
 #ifndef MAP_H
 #define MAP_H
 
-#include "common.h"
-#include "geometry.h"
 
 #ifndef SEEK_SET
 #define SEEK_SET 0
@@ -35,8 +33,8 @@ void map_create(char *name);
 void map_descrs(DESCR_SECTORS *descr, SECTOR_ADDR* addr, SECTOR_ADDR* dflcmd);
 /* Updates map file with descriptors & default command line */
 
-void map_close(void);
-/* closes the map file */
+void map_close(BOOT_PARAMS_2 *param2, off_t here);
+/* closes the map file, write second stage parameters */
 
 void map_add_sector(void *sector);
 /* Adds the specified sector to the map file and registers it in the map
