@@ -24,9 +24,11 @@ typedef unsigned int md_u32;
 typedef unsigned short md_u16;
 typedef unsigned char md_u8;
 
-#include <linux/major.h>
+/*#include <linux/major.h>*/
 #include <sys/ioctl.h>
 
+#define MD_MAJOR MAJOR_MD
+
 /*
  * Different major versions are not compatible.
  * Different minor versions are only downward compatible.
