@@ -1,7 +1,7 @@
 /* partition.c  -  Partition table handling */
 /*
 Copyright 1992-1998 Werner Almesberger.
-Copyright 1999-2002 John Coffman.
+Copyright 1999-2003 John Coffman.
 All rights reserved.
 
 Licensed under the terms contained in the file 'COPYING' in the 
@@ -20,9 +20,9 @@ source directory.
 #include <a.out.h>
 #include <sys/stat.h>
 #include <sys/types.h>
-#include <linux/unistd.h>
+#include <asm/unistd.h>
 #include <limits.h>
-#include <linux/fs.h>
+/*#include <linux/fs.h>*/
 #include <time.h>
 #include "config.h"
 #include "lilo.h"
@@ -73,12 +73,16 @@ int part_nowrite(char* device)
     if (read(fd, bs.sector, sizeof(bs)) != SECTOR_SIZE) pdie("part_nowrite: read:");
     
 /* check for XFS */
-    if (!strncmp("XFSB", bs.sector, 4)) ret=2;
+    if (!strncmp("XFSB", bs.sector, 4)) ret=3;
     
 /* check for NTFS */
     else if (	!strncmp("NTFS", bs.par_d.system, 4)
 		|| strstr(bs.sector,"NTLDR")  ) ret=2;
 
+/* check for HPFS */
+    else if (	!strncmp("OS2", bs.par_d.system, 3)
+		|| strstr(bs.sector,"OS2LDR")  ) ret=2;
+
 /* check for DOS FAT */
     else if (
 	(bs.par_d.bpb.media_descriptor >= 0xF8 || bs.par_d.bpb.media_descriptor == 0xF0)
@@ -95,7 +99,7 @@ int part_nowrite(char* device)
 	    pdie("part_nowrite lseek:");
 	if (SECTOR_SIZE != read(fd, bs.sector, sizeof(bs)) ) pdie("part_nowrite swap check:");
 	if (!strncmp(bs.sector+SECTOR_SIZE-10,"SWAPSPACE2",10)
-	    || !strncmp(bs.sector+SECTOR_SIZE-10,"SWAP-SPACE",10) ) ret=2;
+	    || !strncmp(bs.sector+SECTOR_SIZE-10,"SWAP-SPACE",10) ) ret=3;
     }
 
 /* didn't recognize the superblock type, so assume it is okay */    
@@ -104,7 +108,7 @@ int part_nowrite(char* device)
     close(fd);
 
 /* check for override (-F) command line flag */
-    if (ret==2 && force_fs) {
+    if (ret>1 && force_fs) {
 	fprintf(errstd, "WARNING: '-F' override used. Filesystem on  %s  may be destroyed.\n", device);
 	ret=0;
     }
@@ -180,11 +184,20 @@ void part_verify(int dev_nr,int type)
 
     if (type && !Linux) {
 	fflush(stdout);
+#if 0
 	if (!dos || !nowarn)
 	    fprintf(errstd,"Device 0x%04X: Partition type 0x%02X does not seem "
 	      "suitable for a LILO boot sector\n",dev_nr,part_table[part].sys_ind);
 	if (dos && !nowarn)
 	    fprintf(errstd,"but DOS/Windows/OS2 FAT partitions are now usable.\n");
+#else
+	if (!nowarn) fprintf(errstd,"Warning: partition type 0x%02X"
+		" on device 0x%04X is an odd place for a boot\n  sector.",
+			part_table[part].sys_ind, dev_nr);
+	if (dos && !nowarn) fprintf(errstd,"  A DOS/Windows/OS2 system may be rendered unbootable."
+		"\n  The backup copy of this boot sector should be retained.");
+	if (!nowarn) fprintf(errstd,"\n");
+#endif
 	if (!dos && !cfg_get_flag(cf_options,"ignore-table")) exit(1);
     }
     cyl = part_table[part].cyl+((part_table[part].sector >> 6) << 8);
@@ -560,7 +573,7 @@ void do_activate(char *where, char *which)
 	die("No active partition found on %s",where);
     }
     number = to_number(which);
-    if (number < 1 || number > 4)
+    if (number < 0 || number > 4)
 	die("%s: not a valid partition number (1-4)",which);
     for (count = 1; count <= PART_NUM; count++) {
 	if (lseek(fd,PART_BEGIN+(count-1)*PART_SIZE+4,0) < 0)
@@ -610,6 +623,7 @@ void do_install_mbr(char *where, char *what)
     } else if (*(int*)&buf[PART_TABLE_OFFSET - 6] == 0) {
     	i = st.st_rdev;
     	i %= 271;		/* modulo a prime number; eg, 2551, 9091 */
+    	i += 31;
         srand(time(NULL));	/* seed the random number generator */
         while (i--) rand();
         *(int*)&buf[PART_TABLE_OFFSET - 6] = rand();  /* insert serial number */
