@@ -7,7 +7,7 @@
 #define PARTITION_H
 
 #include <unistd.h>
-#include <linux/unistd.h>
+/*#include <linux/unistd.h>*/
 
 typedef struct _change_rule {
     const char *type;
@@ -24,6 +24,20 @@ typedef struct _change_rule {
 
 #endif
 
+struct partition {
+	unsigned char boot_ind;		/* 0x80 - active */
+	unsigned char head;		/* starting head */
+	unsigned char sector;		/* starting sector */
+	unsigned char cyl;		/* starting cylinder */
+	unsigned char sys_ind;		/* What partition type */
+	unsigned char end_head;		/* end head */
+	unsigned char end_sector;	/* end sector */
+	unsigned char end_cyl;		/* end cylinder */
+	unsigned int start_sect;	/* starting sector counting from 0 */
+	unsigned int nr_sects;		/* nr of sectors in partition */
+};
+
+
 int part_nowrite(char* device);
 /* identify partitions which would be destroyed if the boot block
    is overwritten:
