@@ -1,6 +1,6 @@
 /* probe.c -- BIOS probes */
 /*
-Copyright 1999-2002 John Coffman.
+Copyright 1999-2003 John Coffman.
 All rights reserved.
 
 Licensed under the terms contained in the file 'COPYING' in the 
@@ -9,6 +9,7 @@ source directory.
 */
 
 /*#define DEBUG*/
+#define BITMAP 0	/* change to 1 when do_bitmap is filled in */
 #define VIDEO 1
 
 #include <stdlib.h>
@@ -20,7 +21,8 @@ source directory.
 #include <errno.h>
 #include <sys/stat.h>
 #include <sys/types.h>
-#include <linux/unistd.h>
+#include <asm/unistd.h>
+#include "lilo.h"
 #include "common.h"
 #include "device.h"
 #include "geometry.h"
@@ -73,13 +75,14 @@ static void do_ebda(void);
 static void do_cr_pr(void);
 static void do_help(void);
 static void do_geom(char *bios);
-static void do_bitmap(char *file);
 static void do_geom_all(void);
 static void do_table(char *part);
 #if VIDEO
 static void do_video(void);
 #endif
-
+#if BITMAP
+static void do_bitmap(char *file);
+#endif
 static char dev[] = "<device>";
 
 extern CHANGE_RULE *change_rules;	/* defined in partition.c */
@@ -113,7 +116,9 @@ struct Probes {
 	}
 	list[] = {
 { "help",  do_help,  NULL,  "Print list of -T(ell) options"	},
+#if BITMAP
 { "bitmap=",do_bitmap,"<file>", "Display .bmp file X,Y/color/timer information"},
+#endif
 { "ChRul", do_cr_pr, NULL,  "List partition change-rules"  },
 { "EBDA",  do_ebda,  NULL,  "Extended BIOS Data Area information" },
 { "geom=", do_geom,  "<bios>", "Geometry CHS data for BIOS code 0x80, etc." },
@@ -585,6 +590,10 @@ static void do_cr_pr(void)
 #if VIDEO
 static int egamem;
 static int mode, col, row, page;
+/*
+enum {VIDEO_UNKNOWN, VIDEO_MDA, VIDEO_CGA, VIDEO_EGA, VIDEO_MCGA,
+	VIDEO_VGA, VIDEO_VESA, VIDEO_VESA_800};
+ */
 
 int get_video(void)	/* return -1 on error, or adapter type [0..7] */
 {
@@ -635,7 +644,7 @@ int get_video(void)	/* return -1 on error, or adapter type [0..7] */
     	if (verbose >= 2)
 	    printf("get display combination\n");
 	if ( buf.s.v.vid1A.al==0x1A ) {
-    	    monitor = (buf.s.v.vid1A.bx >> ((verbose>=3)*8) ) & 0xFF;
+    	    monitor = (buf.s.v.vid1A.bx >> ((verbose>=9)*8) ) & 0xFF;
     	    switch (monitor) {
     	      case 1:
     	        adapter = 1;
@@ -681,7 +690,7 @@ int get_video(void)	/* return -1 on error, or adapter type [0..7] */
 	}
     }
     if (verbose>=2)
-    	printf ("mode = 0x%02x,  columns = %d,  rows = %d,  page = %d\n\n",
+    	printf ("mode = 0x%02x,  columns = %d,  rows = %d,  page = %d\n",
     		       mode, col, row, page);
     
     return adapter;
@@ -702,10 +711,6 @@ static char *display[] = { "unknown", "MDA", "CGA", "EGA",
         return;
     }
     
-    if (verbose>=1)
-    	printf ("mode = 0x%02x,  columns = %d,  rows = %d,  page = %d\n\n",
-    		       mode, col, row, page);
-
     printf("%s adapter:\n\n", display[adapter]);
     if (adapter < 3 || (adapter == 3 && egamem < 1) ) {
         printf("No graphic modes are supported\n");
@@ -795,7 +800,7 @@ int bios_device(GEOMETRY *geo, int device)
         
     if (fetch()) return -1;
     
-    if (verbose>=3) printf("bios_dev:  device %04X\n", device);
+    if (verbose>=4) printf("bios_dev:  device %04X\n", device);
     
     match = 0;
     bios1 = -1;		/* signal error */
@@ -810,14 +815,14 @@ int bios_device(GEOMETRY *geo, int device)
 	}
     }
     if (match == 1) {
-    	if (verbose>=2) printf("bios_dev: match on geometry alone (0x%02X)\n",
+    	if (verbose>=4) printf("bios_dev: match on geometry alone (0x%02X)\n",
     		bios1);
 	return (geo->device = bios1);
     }
 
     device &= D_MASK(device);	/* mask to primary device */
     fd = dev_open(&dev,device,O_RDONLY);
-    if (verbose>=3) printf("bios_dev:  masked device %04X, which is %s\n", 
+    if (verbose>=4) printf("bios_dev:  masked device %04X, which is %s\n", 
     			device, dev.name);
     if (lseek(fd, PART_TABLE_OFFSET-8, SEEK_SET)!=PART_TABLE_OFFSET-8)
 	die("bios_device: seek to partition table - 8");
@@ -837,7 +842,7 @@ int bios_device(GEOMETRY *geo, int device)
     }
 #endif
 
-    if (verbose>=3) printf("bios_dev: geometry check found %d matches\n", match);
+    if (verbose>=4) printf("bios_dev: geometry check found %d matches\n", match);
     match = 0;
     bios1 = -1;
     while (--bios >= 0x80) {
@@ -885,9 +890,10 @@ int bios_device(GEOMETRY *geo, int device)
 }
 
 
+#if BITMAP
 static void do_bitmap(char *file)
 {
     printf("Color, Positioning, and Timer information for file:  %s\n", file);
     printf("...<unimplemented>...\n");
 }
-
+#endif
